#!/bin/ksh
##########################################################################
#
#          Licensed Materials - Property of IBM
#   5724-B96
#
#          Copyright IBM Corp. 2006 All Rights Reserved
#          US Government Users Restricted Rights - Use, duplication or
#          disclosure restricted by GSA ADP Schedule Contract with
#          IBM Corp.
#
##########################################################################
#
# Create :
# D29047 This script is to support separate CTIRA_HIST_DIR per instance
#        for oracle and sybase agent on UNIX.
#        Since they're sharing kddexec.sh in candlehome,they have to run
#        this script not to lose collected historical data by applying 
#        Oracle or Sybase agent LA/FP patch.
#

#
# Check to see if CANDLEHOME is set
#

if [ -z "$CANDLEHOME" ]
then
    echo "CANDLEHOME is not set. Get it from the parameter."

    if [ $# -lt 1 ]
    then
        echo "CANDLEHOME is not set. Set CANDLEHOME or provide as a parameter."
        exit 2
    else
        CANDLEHOME=$1
    fi
fi

CURRENTPWD=`pwd`

#
# Run the script for oracle agent.
#
cd $CANDLEHOME/logs

#
# Check to see how many instances had been running
#

LOGS=`ls *_or_*_col.log`

count=`echo $LOGS | wc -w`

if [ $count = 0 ] ;
then
   exit 0
fi

#Find the architecture
CurRegPath=$CANDLEHOME/registry
ArchTbl=$CurRegPath/archdsc.tbl
DynShl=$CANDLEHOME/bin/dynarch.shl

. $DynShl $ArchTbl 2>/dev/null
setBinLibArchs $cArch or

if [ $count = 1 ] ;
then

    #Find the begining index of the instance using '_or_' as the
    #matching pattern
   
    begin=`expr length ${LOGS%*_or_*}`
    begin=`expr $begin + 5`

    #Find the end of the instance using '_col.' as the matching
    #pattern
    end=`expr index "$LOGS" .`
    end=`expr $end - 4`

    #From the index found, calculate the len of the instance name
    #and extract that from the filename
    len=`expr $end - $begin`
    INSTANCE=`expr substr $LOGS $begin $len`

    # Change directory to current hist dir
    cd $CANDLEHOME/$cArch/or/hist

    #COPY the existing history data into the CTIRA_HIST_DIR/<instance>
    #If there is no history data file, do nothing
    if [ ! -d $INSTANCE ]
    then
        mkdir $INSTANCE
    fi

    #Count number of history data file(s), only move if exist
    hist_file_count=`ls KOR* 2>/dev/null | wc -w `
    if [ $hist_file_count -gt 0 ]
    then
        mv KOR* $INSTANCE
    fi

else
    #Delete the history data file if it exists
    cd $CANDLEHOME/$cArch/or/hist
    hist_file_count=`ls KOR* 2>/dev/null | wc -w `
    if [ $hist_file_count -gt 0 ]
    then
        rm KOR*
        echo "Need to remove KOR history files in $CANDLEHOME/$cArch/or/hist"
    fi
fi

#
# Run the script for sybase agent.
#
cd $CANDLEHOME/logs

#
# Check to see how many instances had been running
#
LOGS=`ls *_oy_*_col.log`

count=`echo $LOGS | wc -w`

if [ $count = 0 ] ;
then
   exit 0
fi

#Find the architecture
CurRegPath=$CANDLEHOME/registry
ArchTbl=$CurRegPath/archdsc.tbl
DynShl=$CANDLEHOME/bin/dynarch.shl

. $DynShl $ArchTbl 2>/dev/null
setBinLibArchs $cArch oy

if [ $count = 1 ] ;
then

    #Find the begining index of the instance using '_oy_' as the
    #matching pattern
   
    begin=`expr length ${LOGS%*_oy_*}`
    begin=`expr $begin + 5`

    #Find the end of the instance using '_col.' as the matching
    #pattern
    end=`expr index "$LOGS" .`
    end=`expr $end - 4`

    #From the index found, calculate the len of the instance name
    #and extract that from the filename
    len=`expr $end - $begin`
    INSTANCE=`expr substr $LOGS $begin $len`

    # Change directory to current hist dir
    cd $CANDLEHOME/$cArch/oy/hist

    #COPY the existing history data into the CTIRA_HIST_DIR/<instance>
    #If there is no history data file, do nothing
    if [ ! -d $INSTANCE ]
    then
        mkdir $INSTANCE
    fi

    #Count number of history data file(s), only move if exist
    hist_file_count=`ls KOY* 2>/dev/null | wc -w `
    if [ $hist_file_count -gt 0 ]
    then
        mv KOY* $INSTANCE
    fi

else
    #Delete the history data file if it exists
    cd $CANDLEHOME/$cArch/oy/hist
    hist_file_count=`ls KOY* 2>/dev/null | wc -w `
    if [ $hist_file_count -gt 0 ]
    then
        rm KOY*
        echo "Need to remove KOY history files in $CANDLEHOME/$cArch/oy/hist"
    fi
fi

cd $CURRENTPWD

