#! /bin/ksh
# $Header: /ctcvs/hclroot/src/kdd/kddexec.sh,v 1.3 2004/06/01 17:46:02 smyun Exp $
######################################################################
#                                                                    #
#        Licensed Materials - Property of IBM                        #
#        5724-B96                                                    #
#                                                                    #
#        Copyright IBM Corp. 2005 All Rights Reserved                #
#        US Government Users Restricted Rights - Use, duplication or #
#        disclosure restricted by GSA ADP Schedule Contract with     #
#        IBM Corp.                                                   #
#                                                                    #
######################################################################
#
#    Name    : kddexec.sh
# DDDEV portion of OMA for Distributed databases startup.
#       Run only by CandleDBAgent, not standalone.
#    COPYRIGHT (C) CANDLE CORP  1998  U.S.A.
#
# Usage:  kddexec -d [ALL|all|+ddd,][nosig,]tab,tab...  servers....
#                 -d order,irrelevant
#
#    Debug table  list can contain wildcards, but CandleDBAgent already expanded servers
#
# 1. Usage:  kddexec.sh action serverlist
#    where action is:
#
#         start    start agent/collector
#         stop     stop  agent/collector
#         trace    Update trace options from ko?trac.ctl
#         status   Write collector status to Server_sta.log
#         csh      Set environmentals, open a csh
#         ksh/sh   Set environmentals, open a ksh
#
#    config-file is the name of the configuration file.  If unqualified,
#    assumed to be in KDDCFGDIR.

# 2. Following environmentals must be set: (normally by CandleDBagent)

#    CT_CMSLIST =
#    KDC_PORTS  =
#    CANDLELOG  = log directory
#    CANDLEMISC = ctl directory
#    CANDLECFG  = cfg directory
#    HOST       = this host
#    USER       = my user id
#    PROD       = 2-letter product code
#    RUNFILE    = runfile name in CONFIG

# 3. Hidden environmentals
#    for 0/1 vars, default is opposite value given, var=* means any non-blank value
#
#    KDDCOLL=0     means Do not run kddcoll, but ko?coll instead.
#    kddcoll=0     accepted also if KDDCOLL not set.
#    COLL_USESHM=0 means to force the collector to use named pipes
#                  instead of shared memory.
#    KDDFUSE       Max wait time in seconds before kill -9
#    KDDSLEEP      Watcher task sleep time in seconds.
#    KDDRESTART    Max self-restarts for collector or agent (counted separately)
#    KDDPREFIX     This agent instance runtime override (no localization if on NFS)
#                  Default $CANDLEHOME/tmp/$HOST_O?_$server
#    KDDPREFIX_LOCAL   Localization area override.  Note, unlike KDDPREFIX,
#                  KDDPREFIX_LOCAL should be shared between multiple agents:
#    KDDTOTTY=0    If user has selected (on HP defaulted) "mesg n", which refuses
#                  messages from the Unix "write" command, then do
#                  NOT try to write directly to his terminal.
#    KDDTOTTY=1    If "write" command fails, write directly to terminal
#    KDDTOTTY=*    (default) For HP-UX, KDDTOTTY=1, else 0.

#    KDDLIBLOC=0   Do not localize shared libaries with binaries.  Default=yes.

# IZ05321 CANDLEDBAGENT KILLS NON-ITM PROCESSES IF RUNINFO PIDS ARE REUSED
# IZ05321 start
# 0 means not reused (good), 1 means reused (bad)
PID_REUSE=0
# IZ05321 end

  export COLL_USESHM=${COLL_USESHM:-0}   # TEMP: USE PIPES NOT SHARED MEMORY.
# Determine collector type  0=default off
  (( ${KDDCOLL:=${kddcoll:-0}} )) && whence kddcoll >/dev/null &&
     Collector=kddcoll || Collector=k$PROD\coll

# Assign env default values.
  true ${KDDFUSE=60} ${KDDSLEEP=180} ${KDDRESTART=5}   ${KDDLIBLOC=1}
  export KDDFUSE       KDDSLEEP        KDDRESTART        KDDLIBLOC

  if   [[ $KDDTOTTY != [01] ]]
  then if [ $CANDLEARCH = hp* ]
       then KDDTOTTY=1
       else KDDTOTTY=0
       fi
  fi
  export KDDTOTTY

# Honor legacy variables and values
  true ${KDDPREFIX:=$COLL_PREFIX}
  # called for each server
  Legacy() { true     ${db_local:=$db_pipedir}
             db_local=${db_local#@(N/A|n/a)}     # N/A meant default.
  }

#####################################################################
# FUNCTIONS
#####################################################################

Usage () {
      echo kddexec.sh fatal error:
      echo $*
      echo Usage: kddexec.sh -d [all debug,list] [start stop trace status] Servers
      exit
}

# Runfile + server pid    create an entry
# Runfile - server        delete an entry
# Runfile . server        return an entry's pid, else RC>0

Runfile() {
  typeset line
  cd $CANDLECFG
  case $1 in
       (+) echo $2=$3 >>$RUNFILE
  ;;   (-) sed "/^$2=/d" $RUNFILE >.$RUNFILE &&
             mv -f .$RUNFILE $RUNFILE
           [[ -s $RUNFILE ]] || rm $RUNFILE
  ;;   (.) line=$(grep -s ^$2= $RUNFILE) && echo ${line#*=}
  esac
}

# RunInfofile + hostname prodcode pid server arch    create an entry
# RunInfofile - hostname prodcode pid server arch    delete an entry

RunInfofile() {
  RUNINFOFILE=$CANDLECFG/.ConfigData/RunInfo
  TempInfoFile=$CANDLECFG/.ConfigData/.RunInfo
  TempEntry="$2|$3|$4|$5|$6"

  if [[ ! -a $RUNINFOFILE ]]
  then
     echo "FIELDSEP=|" > $RUNINFOFILE
  fi

  case $1 in
       (+) echo $TempEntry >>$RUNINFOFILE
  ;;   (-) eval $(grep -v $TempEntry $RUNINFOFILE > $TempInfoFile) &&
            mv -f $TempInfoFile $RUNINFOFILE
  esac
}
# IZ05321 CANDLEDBAGENT KILLS NON-ITM PROCESSES IF RUNINFO PIDS ARE REUSED
# IZ05321 start
RunInfoDescriptionfile() {

  # defect 67459 instead of doing ps, create an entry with parameters.
  #PIDDescription=`ps -fp $4 | sed 's/  */ /g' | sed 's/^ //g' | sed 's/ $//g' | cut -d" " -f8- | tail -1`

  RUNINFODESCFILE=$CANDLECFG/.ConfigData/RunInfo_Description
  TempInfoFile=$CANDLECFG/.ConfigData/.RunInfo_Description
  TempEntry="$2.description=$3 $4 $5 $6"
  RemoveEntry="$2"

  case $1 in
       (+) echo $TempEntry >>$RUNINFODESCFILE
  ;;   (-) eval $(grep -v $RemoveEntry $RUNINFODESCFILE > $TempInfoFile) &&
            mv -f $TempInfoFile $RUNINFODESCFILE
  esac
}

RunInfoDescriptionfileRemove() {

  RUNINFODESCFILE=$CANDLECFG/.ConfigData/RunInfo_Description
  TempInfoFile=$CANDLECFG/.ConfigData/.RunInfo_Description
  TempEntry="$4.description=$PIDDescription"
  RemoveEntry="$4"

  grep -v $RemoveEntry $RUNINFODESCFILE > $TempInfoFile
  mv -f $TempInfoFile $RUNINFODESCFILE

}

# This will check to see if the stored description value has the same as the running description
RunInfoDescriptionfileCheck() {

  RUNINFODESCFILE=$CANDLECFG/.ConfigData/RunInfo_Description
  RemoveEntry="$4.description"

  test1=`ps -fp $4 | sed 's/  */ /g' | sed 's/^ //g' | sed 's/ $//g' | tail -1 | cut -d" " -f5 | grep ":"`
  val=$?

  cutVal="-f8-"

  if [[ $val = 0 ]] ; then
    # "it has : in it, we're good"
    foo=bar #bogus filler line
  else
    # "it does NOT have : in it, we're NOT good - need to check further"

    # if we're here, it means that the time is over 24 hours
    # we need to see if this is Linux (in which case the 5th field is "MONDD",
    # or another OS, which means "MON DD" over 5th and 6th fields
    # in which case we index the field we search up one

    # we assume its needed, since all but Linux needs it
    # anything else not in this list we assume needs it
    needed=yes

        case $6 in
        (hp*) needed=yes
        ;; (sol*) needed=yes 
        ;; (aix*) needed=yes 
        ;; (li*)  needed=no
        ;; (ls*)  needed=no
        ;; (lp*)  needed=no
        ;; (lx*)  needed=no
        esac

    # if its needed, we only cut the 9th field on =>, skipping the 8th. Else we just use the 8th=>
    if [[ $needed = "yes" ]] ; then
        # Size is 4 - which means just "MON", so we need to index
        cutVal="-f9-"
    else
        # Size is not 4 - probably bigger - this means that the date is likely "MONDD", like on linux - and we can keep our index OK
        foo=bar #bogus filler line
    fi

  fi

  PIDDescription1=`ps -fp $4 | sed 's/  */ /g' | sed 's/^ //g' | sed 's/ $//g' | cut -d" " $cutVal | sed 's/.*:[0-9][0-9] //g' | tail -1`

# Compare after go command
PIDDescription=`echo $PIDDescription1 | sed 's/.*go //g'` 

  if [ -f $RUNINFODESCFILE ] ; then
    # file exists, continue
  
        # Compare after go command
    storedDescription=`grep $RemoveEntry $RUNINFODESCFILE | cut -d"=" -f2 | sed 's/.*go //g'` 

    # D70931 - Solaris 9 truncates the description in the "ps -ef" output to 
    #          80 characters, so instead of an exact match, check whether the 
    #         ps -ef output is contained in the stored description (i.e. substring) 
    ContainsDesc=`echo $storedDescription | grep "$PIDDescription"`
    # if [[ $PIDDescription = $storedDescription ]] ; then
    if [ "$ContainsDesc" != "" ]; then
        # This is good, descriptions are the same, so we keep the value 0
        foo=bar #bogus filler line
    else
        # This is bad, descriptions are different, so we set the value 1
        PID_REUSE=1
    fi
  else
    #file doesn't exist, value stays 0, it will stop, that's cool
    foo=bar #bogus filler line
  fi

}
# IZ05321 end

#####################################################################
# Define functions used by kddexec.sh, only
#####################################################################

# Dictionary:
#    COLL_PREFIX_LOCAL is passed to k??coll, where it writes .out stuff
#    KDDPREFIX_LOCAL is the user's runtime override, influences coll & agent

#  This routine is called for every agent instance
#  Rules
#  IF   KDDPREFIX_LOCAL = NONE, disables all localization.
#  elif config db_local may be a path or NONE.
#  elif KDDPREFIX_LOCAL = path,  localize at path
#  else default is /tmp/kdd.$USER/$dynamo, but NFS execution

#  Note, these rules are modified by the Legacy rules, which allow
#  1.   in absence of config db_local,  allow db_pipedir
#  2.   db_local=N/A  means ""

#  Also, a KDDPREFIX unconditionally overrides COLL_PREFIX_LOCAL,
#  but not the binaries or ctl files.

Localize() {
  typeset local localbin localrun
  export COLL_PREFIX_LOCAL=          # no localization until proven OK

  # Determine how/where user wants localization per rules above

    if   [[ $KDDPREFIX_LOCAL   = ?(\*)@(none|NONE) ||
            $db_local          = ?(\*)@(none|NONE) ]]
    then return
    elif local=${db_local:-$KDDPREFIX_LOCAL}
         [[ -n $local ]]
    then localbin=$local/cccbin
    else local=/tmp/kdd.$USER  # but no localbin
    fi

    localrun=$local/$Dynamo

    [[ -d $localrun ]] || mkdir -p $localrun ||
    { echo Could not create local temp directory=$localrun
      echo Running from $CANDLETMP
      return 99
    }

  # NOTE, this assumes that all of CANDLEHOME is on the same filesys.
  # To be truly general, we should separately check $CANDLELOG,
  # $CANDLETMP and $(whence kddos)

  # What kind of disk are $CANDLEHOME and $localrun on?  return if cannot tell

  typeset where
  where=$(kddos $Dlog isremote $CANDLEHOME $local) || {
    echo >&2 kddexec: kddos isremote failed, no localization
    return
  }

  case $where in
     (remote?local)            # really   "remote\nlocal"

      # check if there is enough space on the local disk.  result in kb.
      kddos $Dloc fspace $local | read a fsfree x fsused x fstotal x

      if   (( fsfree < 100 ))      # filesys freespace < 100KB
      then echo kddexec:  not enough free space on $local directory
           return
      else COLL_PREFIX_LOCAL=$localrun   # at least for logs
           [[ -n $localbin ]] || return  # no localbin per Rule #4 above
           if (( fsfree > 600 ))
           then echo kddexec: Localizing binaries to $localbin
           else echo Not enough free space on $local for executables
                return
           fi

           # create localbin directory
           [[ -d $localbin ]] || mkdir -p $localbin ||
           { echo   Could not create local bin directory=$localbin
             return
           }
      fi

  ;; (local*)   echo kddexec:  $CANDLEHOME already local disk
                 return
  ;; (*remote)  echo kddexec:  $local is not local disk, running from $CANDLEHOME
                 return
  esac

  # Copy executables.  To avoid making assumptions about where they
  # are, we locate all relevant executables from any dir in the
  # $PATH under $CANDLEHOME.   Exclude wrong collector, debug tools and scripts.
  # Note that this excludes all kdd*.sh setup-related scripts, but includes
  #      all product-specific scripts like koxerr.sh

  progs=$(
    for  dir in $(IFS=:; echo $PATH)
    do   [[ $dir = $CANDLEHOME/* ]] && cd $dir || continue      # picky on dirs
         ls kdd!(coll|test|prob*|trac*|stat|*.sh) $Prod!(col*) $Collect   # 62380
    done 2>/dev/null | sort -u)

  progs=$(whence -p $progs)   # convert to fully-qualified

  #  make a list of shared libs, find them and copy them
  #  For ct99 following needs(1) add libs+=kde, del libs2.sol=pthread
  if (( KDDLIBLOC ))
  then
    typeset libx libs="kra kdc kbb kdsfilt" libs2=
    case $CANDLEARCH in
       (hp*)  libx=sl  libs2=g++\ stdc++
    ;; (sol*) libx=so
    ;; (aix*) libx=a
    ;; (li*)  libx=so
    ;; (ls*)  libx=so
    ;; (lp*)  libx=so
    ;; (lx*)  libx=so
    esac

    # cute kludge:  all shared libs are shipped with execute bit on, so whence can
    # find them as if they were programs

    libs=$(eval PATH=\$$Libpathvar
           for name in $libs $libs2; do whence -p lib$name.$libx; done)
  else libs=
  fi

  kddos $Dloc copy $progs $libs $localbin ||
  { echo copy of programs/libs for localized run failed
    echo using executables from $CANDLEHOME
    return 99
  }

  #        set library link
  # Note: Link is not needed for database agent because it does not run as
  # root.  Only kux on solaris need worry about this.
  # Note: this has one (big) problem. If the /opt/candle link exists and
  # points to the NFS library location, kddos will not change it.   This is
  # the safer way to go but if the NFS volume goes away kux will not be able
  # to load any shared routines (unless UNIX takes care of this condition and
  # preloads all routines from NFS volumes).
  if [[ $CANDLEARCH = sol* && $PROD = ux ]]
  then
    kddos $Dloc setlink /opt/candle $CANDLEHOME/lib/$CANDLEARCH ||
    { echo symbolic link for libraries failed
      return
    }
  fi

  # CTL files:
  # copy regular CTL files, and over-write with overrides
    kddos $Dloc copy $KDDCTLDIR/$Prod*.ctl $COLL_PREFIX_LOCAL &&
                      export COLL_CTL_HOME=$COLL_PREFIX_LOCAL
    typeset ovr_ctl=$KDDLOGDIR/$Prod*.ctl
    test -f $ovr_ctl && kddos $Dloc copy $ovr_ctl $COLL_PREFIX_LOCAL

  # NOW, after all safely copied, change env vars to make it live.
  PATH=$localbin:$PATH
  (( KDDLIBLOC )) && eval $Libpathvar=$localbin:\$$Libpathvar
}

# Write log header
Log_init () { echo "\
                Candle OMA $1 logfile for $Server

Environment strings:
$(env | sort)

----------------------------------------------------------------
$(date) Begin $1 log:\n"
}

#####################################################################
# Define functions used here and in the go-script
#####################################################################

Common_Functions='
Say () { # Write this message to user and both logs:
  typeset my msg="kddexec: $*" date=$(date "+%d%b%y %T")
  echo "$msg"
  for my in agt col
  do  echo "$date $msg" >>$CANDLELOG/$Dynamo\_$my.log
  done
}

Final () { # Watch processes complete, force-clean if needed,  Recursive once.
  typeset pid temp list=$*
  typeset -i timer=0 check=2
  Say waiting up to $KDDFUSE seconds...
  while (( (timer+=check) <= KDDFUSE ))
  do  sleep $check
      temp=$list list=    # rebuild list each pass
      for pid in $temp
      do  kill -0 $pid && list=$list\ $pid ||
            Say process $pid down after $timer seconds
      done
      [[ -z $list ]] && break
  done

  if  [[ -n $list ]]
  then Say waited $KDDFUSE seconds, force-killing pids: $list
       kill -9 $list
  fi
  [ $kddfinal ] && return

  # next, kill any orphan sub-tasks, or anything I inherited.

  list=
  ps -fu ${kdduser:=$USER} | while read x pid ppid x x x x cmd args
  do if [[ $ppid = 1  && $cmd = k@(or|oy|ox|dd)!(coll*|agent)
        || $ppid = 1  && $cmd = k@(oy12|oy15)col
        || $ppid = $$ && $cmd = !(ps|<defunct>) ]]
     then Say "killing $pid $cmd $args (orphan)"
          [[ $cmd = ko?agent ]] && kill -INT $pid || kill $pid
          list=$list\ $pid
     fi
  done

  if [[ -n $list ]]
  then kddfinal=1 Final $list    # wait and force-kill anything left
       Say all secondary processes down
  else Say all processes down cleanly
  fi
  Core_check

  # Clean shared memory.  File=agt_pid col_pid mem sem mem sem
    file=$CWD/$COLL_SERVERID\_$COLL_VERSION\_$HOST
    if [[ -r $file ]]
    then set $(<$file)
         Say cleaning shared memory id: $3 $5, semaphores $4 $6
         ipcrm -m$3 -s$4 -m$5 -s$6
    fi
}'    # here ends long 'string' defining common functions

eval "$Common_Functions"  # define here

#####################################################################
# Functions for go-script, only.  But coded here so they can be written
# without quoting every variable reference.
#   code note: Main go-script runs Hush for checks

Go_Functions='
Talk() { exec >>Talk; }

Hush() {
   # IY98181 rewrote the use of "test" to use "if-then-else" explicitly
   # The "if-then-else" code should work for all platforms, but for now 
   # will be just Solaris so as to limit testing needed
   if [[ $CANDLEARCH = sol* ]] ; then
      exec >&-                # close stdout (which is the file Talk
      if [ -s Talk ] ; then   # if the length of Talk is > 0
         echo "(press enter)" >>Talk  # add message to end of Talk
         if [ "$USER" = "" ] ; then
            USER="root"      # set default USER value if not set
         fi
         # will corrupt /var/adm/utmpx if multiple users are 
         # logged. Solaris writes the following to stdout:
         #    root is logged on more than one place.
         #    You are connected to "pts/5".
         #    Other locations are:
         #    pts/6
         # and if stdout is not redirected, it corrupts utmpx (not sure why)
         write $USER <Talk >/dev/null 2>&1    # say it with write
         if [ "$?" != "0" ] ; then # if write failed
            if [ $KDDTOTTY = 1 ] ; then    # if flag set to echo directly
               # note: because single quotes cannot be used, due to this 
               # source code being in single quotes, we need to use read instead of awk
               ttyrc=`who | grep "^$USER" | read x tty x`  # get the tty from who
               if [ -w /dev/$tty ] ; then    # if tty is writable
                  echo "$(<Talk)" >/dev/$tty
              fi
            fi
         fi
         rm Talk
      fi
   else
      exec >&-                              # close Talk
      [[ -s Talk ]] &&                      # if anything to say...
      echo "(press enter)" >>Talk &&
      write $USER <Talk 2>&- ||           # say it with write, but if cant...
      { [[ $KDDTOTTY = 1 ]] &&                  # and user said OK...
           who | grep ^$USER | read x tty x &&   # and logged on...
	   [[ -w /dev/$tty ]] &&               # and I can write...
	   echo "$(<Talk)" >/dev/$tty        # say directly
      } && rm Talk                      # if either way worked, delete file
   fi
}

Check() {
# kill -0 $2 && return    # normal case, its OK
# IZ01194 check the process is real collector and agent
 processes=`ps -e | grep $2`
 echo "$processes" >processes.temp
 while read pid x x cmd
 do
    if [[ $pid = $2 && $cmd = k@(or|oy|o9|o8)@(coll|agent) ]]
    then
        kill -0 $2 && return    # normal case, its OK
    fi
    if [[ $pid = $2 && $cmd = k@(oy12|oy15)col ]]
    then
        kill -0 $2 && return    # normal case, its OK
    fi
 done <processes.temp
 rm processes.temp
# IZ01194 end

 Talk
 typeset me=$1\ $Parms
 if (( ($1_restarts+=1) <= KDDRESTART))
 then  eval typeset n=\$$1_restarts
       Say restart $n of $KDDRESTART for $me
       $1   R=$n
 else  Say "cannot restart $me, restart limit of $KDDRESTART exceeded.  " \
           "Shutting down..."
       kill $$
 fi
}

# Note solaris argv string is at the end.
# Also, AIX sometimes adds a garbage 1st char to c++ dumps (aix 4.3 adds more junk)

Core_check() {
  [[ -f core ]] || return
  typeset kxx=k[do][dryx]    # RE (not ERE) fragment for kdd, kor, etc
  case $CANDLEARCH in
     (sol*)  set -- $(tail -4000c core | strings | grep "^$kxx")
  ;; (hp*)   set -- $(strings core | head        | grep "^$kxx")
  ;; (aix*)  set -- $(strings core | head -99 | egrep "^.?$kxx")
             [[ $1 = ?k* ]] && set ${1#?}   # get rid of leading ?
  ;; (li*)   set -- $(strings core | head        | grep "^$kxx")
  ;; (ls*)   set -- $(strings core | head        | grep "^$kxx")
  ;; (lp*)   set -- $(strings core | head        | grep "^$kxx")
  ;; (lx*)   set -- $(strings core | head        | grep "^$kxx")
  esac
  typeset new=core.${1:-unknown}.$(date "+%h%d-%H.%M.%S")
  mv core $new
  Talk
  Say core of $1 in $PWD renamed to $new
}

Report() {
  Talk
  Say "report for $*, pid=$$
Pids:      Collector=$col\\t\\tAgent=$agt
Restarts:  Collector=$Collector_restarts\\t\\tAgent=$Agent_restarts
Checks:    $Snoozes"
  Hush
}'

#####################################################################
# MAIN PROGRAM
#####################################################################

# Interpret options

  while getopts d: opt
  do case $opt in
        (d)   Debug=$OPTARG
     ;; (?) Usage  Invalid option: -$opt
     ;; esac
  done
  shift OPTIND-1
  Action=$1 shift
  Servers=$*

# Short version of $CANDLEHOME to show user on ps command.  If
# $CANDLEHOME begins with my home directory, replace it with ~$USER

  [[ $CANDLEHOME = $HOME/* ]] && C_home=~$USER/${CANDLEHOME#$HOME/} ||
                                 C_home=$CANDLEHOME

# Server-independent vars
  Prod=k$PROD

  export COLL_HOME=$CANDLELOG
  export COLL_CTL_HOME=$CANDLEMISC  # overridable in Localization code
  export COLL_VERSION=$Prod
  #  D34063 - if COLL_HOSTNAME has been set in the .config file, don't over-ride it
  if [ -z "$COLL_HOSTNAME" ] ;  then
     export COLL_HOSTNAME=$HOST
  fi

# Resolve debug

  typeset -l trace=  tables=
  Dparm=

  for debug in $(IFS=,; echo $Debug)
  do  case $debug in
         (ALL)   DParm=-dddd           trace=k   break
                 echo "All Agent/Collector functions will be traced in detail"
      ;; (all)   DParm=${DParm:--dddd}   trace="kra kdd $Prod"
                 echo "All tables will be traced, debug information collected"
      ;; (debug) DParm=${DParm:--dddd}
                 echo "Agent/Collector debug information will be collected"
      ;; (+(d) ) DParm=-$debug
      ;; (nosig) export KBB_SIG1="-dumpoff -asyncoff"
                 echo "Agent signal handling disabled"
      ;; (*)     tables=$tables\ $debug
      esac
  done

  if [[ -n $DParm ]]
  then         true ${COLL_WRAPLINES=100000} ${COLL_NUMOUTBAK=9}
               export COLL_WRAPLINES           COLL_NUMOUTBAK
               Dloc=-dddd    # trace kddos localization calls
  fi

  # convert tables to a set of trace options, unless trace= already set
  # deeply desired enhancement:  also rig ko?trac.ctl to trace this table

  if   [[ -z $trace && -n $tables ]]
  then tempdir=$CANDLETMP/$$
       mkdir $tempdir 2>/dev/null
       cd    $tempdir

     # Make a file for each table containing the ras1 tracing options.
       cat $KDDCTLDIR/$Prod\name.ctl | while read table ira
       do  echo kra$ira $Prod$ira >$table
       done

     # Read files for customer's requested tables, complain if one not there, add kdd.
       echo "Tables traced in agent: " $tables
       trace=kdd\ $(cat $tables) ||
          echo "-d tables... contained a bad table name, ignored.\n" \
               "Good table names are:\n\n" * \\n
       cd $OLDPWD
       rm -r $tempdir
  fi

  [[ -n $trace ]] &&                 # Convert trace into KBB_RAS1 form
    export KBB_RAS1="($(for me in $trace; do echo "(UNIT:$me ALL)\c"; done))"

for Server in $Servers
do ( # Each server gets its own subshell
     # Note, for Flinstone II, we drop the requirement that Config exist.

     # Source the config file, complain if not a valid shell script
     # Creates COLL_ vars

  Dynamo=$HOST\_$PROD\_$Server   # Dynamogically correct name

  Config=$CANDLECFG/$Dynamo.cfg
  trap 'echo kddexec.sh error: $Config is not a valid ksh shell script' EXIT
  . $Config || exit  # force the trap if RC != 0
  trap - EXIT

# forget this if server not configured
  [[ $db_installstatus = CONFIGSUCCESS ]] ||
     exec echo "Bypassing server $Server because CandleDBConfig status is" \
               $db_installstatus

  # Check db_home, add DBMS bin directory to path
  [[ -d $db_home ]] ||
  { echo db_home=$db_home not found,
    echo Suggest you rerun CandleDBConfig.  Skipping server $Server
    continue
  }

  PATH=:$db_home/bin:$PATH


  # set the database libraries in the library path variable for shared libraries.
  case $CANDLEARCH in
       (hp*)  Libvar=SHLIB_PATH
    ;; (sol*) Libvar=LD_LIBRARY_PATH
    ;; (aix*) Libvar=LIBPATH
    ;; (li*)  Libvar=LD_LIBRARY_PATH
    ;; (ls*)  Libvar=LD_LIBRARY_PATH
    ;; (lp*)  Libvar=LD_LIBRARY_PATH
    ;; (lx*)  Libvar=LD_LIBRARY_PATH
  esac

  # if var first exists dblib is first, else last.  $first must be null
  unset first
  case $db_type in
       (koy*)        dblib=$db_home/lib:$SYBASE_OCS/lib
    ;; (kor*)        # D52842 - make lib32 last is this is 64bit arch
                     # get the bitmode (32bit or 64bit) from CANDLEARCH
                     # for example is CANDLEARCH=aix523, this returns 3
                     ARCHBIT=`echo $CANDLEARCH | sed 's/^.*\(.\)$/\1/'`
                     if [ "$ARCHBIT" = "3" ] ; then
                        dblib=$db_home/lib32:$db_home/lib:$db_home/lib64
                     else
                        dblib=$db_home/lib:$db_home/lib64:$db_home/lib32
                     fi
    ;; (kox*) first= dblib=$db_home/lib:$db_home/lib/esql
  esac
  if [[ $CANDLEARCH = sol* &&  $db_type = koy* ]]
  then
       first=$db_home/lib
  fi      

  #            $first exists            $first not exit  :$dblib is text
  eval $Libvar=${first+$dblib:}\$$Libvar${first-:$dblib}

  # if informix then we want to set DBLANG incase national language support is
  # required.  Note: the search if from the best choice to anything there.
  # and stops on the first "ls" that finds the .iem files.
  if [[ $db_type = kox* ]]
  then
    DBLANG=$( cd $INFORMIXDIR
         ( ls -1d $DBLANG/*.iem ||
           ls -1d msg/en_us/0333/*.iem ||
           ls -1d msg/en_us/+([0-9])/*.iem ||
           ls -1d msg/*/+([0-9])/*.iem ||
           ls -1d msg/*/*/*.iem ||
           ls -1d msg/*/*.iem ) 2>&- | head -1 )
    export DBLANG=${DBLANG%/*}
  fi

  # exception for Informix:  resolve ONCONFIG if not provided
  # this is not bulletproof, but koxstat needs an ONCONFIG, not critical that
  # it be the right one.   If nothing else, onconfig.std, which will certainly
  # be the oldest one, will suffice.
  # final resolution for this will be the unified environment-handler

  if   [[ $Prod = kox && -z $ONCONFIG ]]
  then cd $INFORMIXDIR/etc
       maybe=onconfig.$INFORMIXSERVER
       if   [[ -a $maybe ]]
       then export ONCONFIG=$maybe
       else set -- $(ls -t onconfig.*)    # get most recently modified
            export ONCONFIG=$1
       fi
       cd $OLDPWD
       export ONCONFIGsrc=kddexec         # so logs reflect this
  fi

  # Following environment variables are set for the collector
  export COLL_SERVERID=$db_sid
  export COLL_LOGONID=$db_login
  export COLL_PASSWORD=$db_password
  export COLL_VERSION=$db_type
  export COLL_DBHOME=$db_home
  export COLL_DBVERSION=$db_ver
  export COLL_INITFILE=$db_initfilename
  # IY92195  COLL_DISABLE_CURSORS setting
  if [[ $Prod = kor ]]
  then
    export COLL_DISABLE_CURSORS=$db_extparms
  fi
  [[ $Prod = kor ]] && COLL_DBVERSION=$(korrel)     # for KORDICT.CTL
  # for stop/trace/etc, source runtime context script, if any
    Context=$CANDLELOG/$Dynamo.context
    if [[ $Action = @(stop|status|trace) ]]
    then [[  -r $Context ]] && . $Context
    else # Interpret env vars to figure out where things should go, record for future
      Legacy
      Localize   # Sets COLL_PREFIX_LOCAL if localization successful

      export COLL_PREFIX_LOCAL=${KDDPREFIX:-${COLL_PREFIX_LOCAL}}
      export COLL_PREFIX=${KDDPREFIX:-$CANDLETMP/$Dynamo}
      CWD=${COLL_PREFIX_LOCAL:-$COLL_PREFIX}

      echo >$Context \
         export COLL_PREFIX=$COLL_PREFIX COLL_PREFIX_LOCAL=$COLL_PREFIX_LOCAL \
                CWD=$CWD \# Agent runtime CWD

      for dir in $COLL_PREFIX $COLL_PREFIX_LOCAL
      do  [[ -d $dir ]] || mkdir -p $dir || Usage Could not create $dir directory
      done

      if   [[ -n $COLL_PREFIX_LOCAL ]]      # may have value now but not before
      then linkname=$CANDLELOG/$Dynamo.runlink
           rm -f $linkname 2>/dev/null
           ln -s $COLL_PREFIX_LOCAL $linkname
           export COLL_OUT=$COLL_PREFIX_LOCAL
           # Removed COLL_HOME=$COLL_PREFIX_LOCAL now that 07 collector honors
           # COLL_OUT.
      else export COLL_OUT=$CANDLELOG
      fi
    fi

  # Decide system type ( 32bit: return 0, 64bit:return 1 )
  if [[ $CANDLEARCH = sol* ]]
  then  systype=$(kddos syschk)
  else  systype=0
  fi
  export COLL_SOL_64=$systype

  systype=0

  if [[ $CANDLEARCH = hp* ]]
  then  systype=$(kddos syschk)
  else  systype=0
  fi
  export COLL_HP_64=$systype

  #
  # IY89037. Only for the solaris 64, define LD_LIBRARY_PATH_64
  #
  if [[ $CANDLEARCH = sol* &&  ${COLL_SOL_64} = 1 ]]
  then
       export LD_LIBRARY_PATH_64=$LD_LIBRARY_PATH:$LD_LIBRARY_PATH_64
  fi


  # d29047 - start
  # Set CTIRA_HIST_DIR per instance
  #
  export CTIRA_HIST_DIR=$CTIRA_HIST_DIR/$COLL_SERVERID

  if [ ! -d $CTIRA_HIST_DIR ]
  then
    mkdir -p $CTIRA_HIST_DIR
  fi
  # d29047 - end

case $Action in
  (start) cv=            # Collector version-suffix

  # Define parallel objects for Agent and Collector
    for me in agent.agt collector.col
    do xxx=${me#*.} me=${me%.*}
       # Rename old log file, initialize new one
         logfile=$CANDLELOG/$Dynamo\_$xxx.log
         [[ -f $logfile ]] &&
         mv -f $logfile ${logfile%log}lg1 2>/dev/null
         Log_init $me >$logfile
       eval log_$xxx=$logfile  # for script
    done

    cd $CWD
    rm Talk 2>/dev/null  # in case left over from earlier

  # Write the go-script.  Note that $parms in the script are
  # substituted when the script is written, \$parms when executed
  # Known problems: -ALRM doesn't end running sleep task. OK.
  if [[ $PROD = or ]]
  then
      if [[ $COLL_DBVERSION = 8* ]]
      then Collector=ko8coll
      elif [[ $COLL_DBVERSION = 9* ]]
      then Collector=ko9coll
      else
           Collector=korcoll
      fi
  elif [[ $PROD = oy ]]
  then
      # 62380
      if [[ $COLL_DBVERSION = 11.* ]]
      then Collector=koy12col
      elif [[ $COLL_DBVERSION = 12.* ]]
      then Collector=koy12col
      else
           Collector=koy15col
      fi
  fi

    [[ -f go ]] && rm -f go
#   -------- write go script ---------------------------------------------------------
    if cat <<! >go && chmod +x go
#!/bin/ksh
# Note, do not use this script to start the OMA.  Use CandleDBagent
# 3 levels of interrupt:
#   ALRM=wake up, run check now.
#   USR1=answer sender with report
#   TERM=shutdown cleanly
Collector() { $Collector$cv  $Server \$$ $DParm \$1 >>$log_col 2>&1 & col=\$!; }
Agent    () { ${Prod}agent   $Server \$$ $DParm \$1 >>$log_agt 2>&1 & agt=\$!; }

Collector
Agent

trap 'kill -s ALRM \$!' ALRM

trap 'Report $Server
      kill -s ALRM \$!' USR1

trap 'trap "echo OMA for $Server termination in progress..." TERM
      Talk
      Say "OMA for $Server received TERM signal
           Stopping Collector=\$col  Agent=\$agt"
      kill  -TERM \$col
      kill  -$( [[ $CANDLEARCH = hp* ]] && echo TERM || echo INT) \$agt
      Hush
      Talk
      Final \$col \$agt
      Say OMA complete
      Hush
      exit' TERM

$Common_Functions
$Go_Functions

typeset -i Snoozes=0 Collector_restarts=0 Agent_restarts=0
while true
do    ((Snoozes+=1))
      sleep $KDDSLEEP & wait \$!
      Core_check
      Check Collector \$col
      Check Agent     \$agt
      Hush
done
!
#   -------- end of write go script -----------------------------------------------
     then if [ -f nohup.out ] ; then
            rm nohup.out
          fi
          # D53784 - use kddsignl instead of go
          # nohup go $Server $C_home <&- >/dev/null &
          cp ${CANDLEHOME}/${CANDLEARCH}/bin/kddsignl .
          tempPWD=${PWD}
          ${PWD}/kddsignl k${PROD}agent $Collector $Server &
          Runfile + $Server $!
          echo "Collector and Agent started for $Server"
          RunInfofile + $HOST $PROD $! $Server $CANDLEARCH
# IZ05321 CANDLEDBAGENT KILLS NON-ITM PROCESSES IF RUNINFO PIDS ARE REUSED
# IZ05321 start
# defect 67459 instead of doing ps, create an entry with parameters.
#          RunInfoDescriptionfile + $HOST $PROD $! $Server $CANDLEARCH
           RunInfoDescriptionfile + $! ${tempPWD}/kddsignl k${PROD}agent $Collector $Server
# IZ05321 end
     else Say  "Start for $Server to $CMS failed, could not write to" $PWD/go
          return 99
     fi

  ;; (stop)  

# IZ05321 CANDLEDBAGENT KILLS NON-ITM PROCESSES IF RUNINFO PIDS ARE REUSED  
# IZ05321 start
# We want to call the check method to determine if the PIDs for this process
# is the same or different. If it's different, we want to skip the stop

            checkPID=$(Runfile . $Server)
            RunInfoDescriptionfileCheck - $HOST $PROD $checkPID $Server $CANDLEARCH

        if [[ $PID_REUSE = 1 ]] ; then
            # This is bad, we cannot continue
                echo "kddexec: Can't continue with the stop";
                echo "kddexec: stop request for PID $checkPID is not acceptable" >>  $CANDLELOG/agentstopfail.log

        else
            # This is good, descriptions are the same, so we keep the value 0
                echo "kddexec: Continue with the stop";

                if   pid=$(Runfile . $Server) && kill -0 $pid
                then Say "normal stop: kill $pid"
                    kill -TERM $pid
                 # D53784 - moved the stop message from go/kddsignl to here
                 stop_timer=0
                 stop_check_interval=1
                 stop_check_max=60
                 while (( (stop_timer+=stop_check_interval) <= stop_check_max))
                 do  sleep $stop_check_interval
                     stop_rc=`ps -ef | grep -v "grep" | grep "$Server $pid" >/dev/null 2>/dev/null`
                     stop_rc="$?"
                     if [ "$stop_rc" != "0" ] ; then
                        #Say Collector and Agent for $Server pid=$pid shutdown gracefully after $stop_timer seconds
                        Say Collector and Agent for $Server shutdown gracefully
                        break
                     fi
                 done
                 if [ "$stop_rc" = "0" ] ; then 
                    Say Collector and Agent for $Server pid=$pid was not shutdown after $stop_timer seconds
                 fi
                    Runfile - $Server
                    RunInfoDescriptionfileRemove - $HOST $PROD $checkPID $Server $CANDLEARCH
                else Say "stop request, no OMA for $Server running"
                fi
                [[ -a $Context ]] && rm $Context
                RunInfofile - $HOST $PROD $pid $Server $CANDLEARCH

        fi

# IZ05321 end
            
  ;; (status) if   pid=$(Runfile . $Server) && kill -0 $pid
              then kill -s USR1 $pid        # Signal go-task to do report
                   cd $CWD
### cores ###          kddstat | tee -a $CANDLELOG/$Server\_col.stat
              else echo "No agent for $Server currently running"
              fi
  ;; (trace)      Say  Refreshing trace options for $Server
                  cd $CWD
                  kddtrace
  ;; (csh)        echo Entering kddexec subshell for $Server
                  csh
  ;; (ksh)        PS1="kddexec $Server ${PS1:-> }" ksh
  ;; (*)  Usage internal error, Action=$Action
  esac
  )
done

# $Log: kddexec.sh,v $
# Revision 1.3  2004/06/01 17:46:02  smyun
# SR1-47110691 Oracle10g support
#
# Revision 1.2  2003/11/10 16:19:23  mevan
# from ctvc
#
# Revision 1.1.1.1  2003/11/10 16:10:43  mevan
# from ctvc
#
# 
#    Rev 1.25.1.7   12 Dec 2000 17:33:34   SELEE
# 300: 1-24H0G To support HP11 64bit
# 
#    Rev 1.25.1.5   24 Oct 2000 11:35:14   SELEE
# 300: (1-24H0K) To support solaris64bit
# 
#    Rev 1.25.1.4   10 Oct 2000 08:59:06   SELEE
# 300: 1-24H0K To decide the machine type, addkddos syschk 
# 
#    Rev 1.25.1.3   29 Feb 2000 15:25:08   SELEE
# 300: (657950) For sybase11.5 on solaris 
# 
#    Rev 1.25.1.1   17 Nov 1999 10:40:50   SELEE
# 300: 651472 Fail to koysql server connection
# 
#    Rev 1.25.1.0   10 Aug 1999 12:22:42   GBECK
# 300: 636872 Customer Sybase libs last in LIBPATH
#
#    Rev 1.25   09 Jun 1999 09:15:36   GBECK
# 200: (608016) if !-d $db_home, suggest CandleDBConfig, not abort but skip server
#
#    Rev 1.24   31 May 1999 23:30:00   GBECK
# 200: SYBLIB1ST Sybase, Informix dblib 1st, Oracle last
#
#    Rev 1.24   29 May 1999 20:34:50   GBECK
# 200: SYBLIB1ST Sybase/lib 1st, rest last
#
#    Rev 1.22   18 May 1999 17:26:50   GBECK
# 200: 99R1ENV 99r1f2 installer changes
#
#    Rev 1.21   03 May 1999 16:05:14   JTRIV
# 200: 609829 Removed * from the libx lib extention
#
#    Rev 1.20   27 Apr 1999 14:00:16   GBECK
# 200: (FIXPAK07) Removed COLL_HOME=$COLL_PREFIX_LOCAL since 07 collector honors $COLL_OUT
#
#    Rev 1.19   23 Apr 1999 11:06:12   GBECK
# 200: KBBRAS1 fix KBB_RAS1, COLL_DBVERSION=$(korrel)
#
#    Rev 1.18   20 Apr 1999 17:38:22   TCAME
# 200: FIXPACK07 missing space in test condition -a $maybe]]
#
#    Rev 1.17   20 Apr 1999 13:31:02   GBECK
# 200: (FIXPAK07) Always set ONCONFIG
#
#    Rev 1.16   29 Mar 1999 04:44:04   GBECK
# 200: FIXFIX07 Bulletproofed kddos isremote calls
#
#    Rev 1.16   29 Mar 1999 04:41:40   GBECK
# 200: FIXFIX07 Bulletproofed kddos isremote calls, localbin
#
#    Rev 1.16   29 Mar 1999 04:36:52   GBECK
# 200: FIXFIX07 Bulletproofed kddos isremote calls
#
#    Rev 1.15   26 Mar 1999 09:59:36   GBECK
# 200: COREID Tweak core regognition for AIX 4.3
#
#    Rev 1.14   23 Mar 1999 11:08:26   GBECK
# 200: FIXPAK07 core.unknown for koxagent on aix
#
#    Rev 1.13   18 Mar 1999 18:57:18   TCAME
# 200: FP07 Export DBLANG if informix
#
#    Rev 1.12   09 Mar 1999 14:18:20   TCAME
# 200: Merge from version 200.98R2, revision 1.6.1.12
#
# >>>>>>>>>>  Q:\DD\MISC\KDDEXEC.SH  Rev 1.6.1.12
#    Rev 1.6.1.12   09 Mar 1999 10:49:28   TCAME
# 200.98R2: FIXPACK2 add database libs to system lib path variable
#
#    Rev 1.6.1.11   24 Feb 1999 10:20:00   GBECK
# 200.98R2: KOXERR.SH Improved localize lib, bin selection comments
#
#    Rev 1.6.1.10   17 Feb 1999 16:22:22   GBECK
# 200.98R2: (FIXPAK) Completed localization clean-up
#
#    Rev 1.6.1.9   12 Feb 1999 16:05:56   GBECK
# 200.98R2: FIXPAK Temp save of NFS logic
#
#    Rev 1.6.1.8   12 Jan 1999 13:11:44   GBECK
# 200.98R2: KILL TEMP--DO NOT PROMOTE
#
#    Rev 1.6.1.7   12 Nov 1998 23:18:22   TCAME
# 200.98R2: (FIXPACK) run though full localization for each db instance
#
#    Rev 1.6.1.6   09 Nov 1998 12:29:56   TCAME
# 200.98R2: (FIXPACK) set default for COLL_LOCAL_PREFIX to N/A
#
#    Rev 1.6.1.5   06 Nov 1998 00:06:14   TCAME
# 200.98R2: (FIXPACK) fixes to localization functions
#
#    Rev 1.6.1.4   13 Oct 1998 16:00:06   TCAME
# 200.98R2: (FIXPACK) more fixpack changes
#
#    Rev 1.6.1.3   18 Sep 1998 12:55:18   TCAME
# 200.98R2: FIXPACK kddunx.c / more chnages for NFS outage toleration
#
#    Rev 1.6.1.2   28 Aug 1998 15:55:52   GBECK
# 200.98R2: -DDDD fix ${Dparm:--dddd}
#
# <<<<<<<<<<  Q:\DD\MISC\KDDEXEC.SH  Rev 1.11
#    Rev 1.11   29 Jan 1999 14:44:48   GBECK
# 200: 608017 Strip leading /tmp_mnt/net from db_home
#
#    Rev 1.10   27 Jan 1999 16:00:54   GBECK
# 200: STOPSIG Added -d nosig, corrected case of (UNIT:kxx ALL)
#
#    Rev 1.9   12 Jan 1999 19:18:40   GBECK
# 200: 602658 Sybase version extract improvements
#
#    Rev 1.8   12 Jan 1999 13:45:54   GBECK
# 200: Merge from version 200.98R2, revision 1.6.1.1
#
# ==========
#    Rev 1.6.1.1   02 Jul 1998 18:21:28   GBECK
# 200.98R2: DESPARATE KDDCOLL=0 to use koyagent
#
#    Rev 1.6.1.0   02 Jul 1998 12:09:56   GBECK
# 200.98R2: 538874 BACK TO PIPES via COLL_USESHM=0
#
#    Rev 1.7   28 Aug 1998 15:58:38   GBECK
# 200: -DDDD fix ${Dparm:--dddd}
#
#    Rev 1.6   12 May 1998 14:08:32   GBECK
# 200: IRAHIST Eliminate CTIRA_HIST_DIR, change .lck to .LCK
#
#    Rev 1.5   01 May 1998 03:00:00   GBECK
# 200: (BUGS) Dynamo Leonardo rewrite
#
#    Rev 1.4   13 Mar 1998 18:43:56   GBECK
# 200: (BUGS) Improved Talk()
#
#    Rev 1.1.1.29   13 Mar 1998 13:42:42   GBECK
# DD120: (BUGS) Improved Talk()
#
#    Rev 1.1.1.28   11 Mar 1998 11:17:36   GBECK
# DD120: (BUGS) Kill all but HP agents with -INT
#
#    Rev 1.1.1.27   30 Dec 1997 16:46:02   GBECK
# DD120: (SETUP) Bugs fixes
#
#    Rev 1.1.1.26   16 Dec 1997 17:17:16   GBECK
# DD120: (SETUP) Bugs Bunny knows What's Up Doc!
#
#    Rev 1.1.1.22   24 Oct 1997 06:55:32   GBECK
# DD120: (SETUP) KDDCOLL support correction.
#
#    Rev 1.1.1.21   21 Oct 1997 16:45:48   GHSU
# DD120: (GHSU) use whence to check kddcoll existence
#
#    Rev 1.1.1.20   15 Oct 1997 16:03:00   GHSU
# DD120: (GHSU) make kill k*sql only happen in collector shutdown
#
#    Rev 1.1.1.19   15 Oct 1997 15:57:04   GHSU
# DD120: (GHSU) add in kddcoll child process clean up
#
#    Rev 1.1.1.18   13 Oct 1997 10:06:18   GHSU
# DD120: GHSU added in kddcoll support and delete CTIRA_HOSTNAME
#
#    Rev 1.1.1.17   18 Aug 1997 16:49:48   GBECK
# DD120: 515840 nohup on process start
#
#    Rev 1.1.1.16   24 Jul 1997 16:40:46   GBECK
# DD120: 522277 ORACLE_HOME as is, CTIRA_HIST_DIR support
#
