#!/bin/ksh
######################################################################
#                                                                    #
#        Licensed Materials - Property of IBM                        #
#        5724-B96                                                    #
#                                                                    #
#        Copyright IBM Corp. 2005 All Rights Reserved                #
#        US Government Users Restricted Rights - Use, duplication or #
#        disclosure restricted by GSA ADP Schedule Contract with     #
#        IBM Corp.                                                   #
#                                                                    #
######################################################################
# Env needs:  CANDLEHOME, CANDLETMP, CT_CMSLIST, KDC_PORTS, CANDLETBL

Null=/dev/null
whence nawk >$Null && nawk=nawk || nawk=awk    # nawk not exist on HP
MyPID=$$
alias rm='rm -f 2>&-'

export COLL_USESHM=0

IVP_go () {
  My_pid=$$        # Remember for kill
  cd $CANDLEHOME
  echo Verifying OMA for Distributed databases installation at \
       "\n   $CANDLEHOME"
  echo "Press Ctl-C at any time to return to configuration menu\n"

# 0.   Set up IVP working directory
  export IVP=$CANDLETMP/kddivp
  mkdir -p $IVP 2>$Null

# 2.  Verify Unix access to common directories
  Check write logs config tmp
  Check read  bin

  typeset This=CMS

# 2.5  Figure out what CMS I'm talking to
  # Translation:
  #    host is contents of file named in KDC_GLBSITES, w/o leading 'ip:'
  #    port is $KDC_PORTS w/o leading '(2 135 ', trailing ')'
  #    CMS name is $CTIRA_CELL_NAME

  host=${CT_CMSLIST#*:}
  CMShost=$(echo ${host%%;*} | awk -F: '{ print substr($1, $2) }')
  # D25589 - this spits out the environment if KDC_PORTS is null, which is ugly,
  #          so skip it if KDC_PORTS is null
  if [ "$KDC_PORTS" != "" ]
  then
     set $(IFS='()'; echo $KDC_PORTS)
  fi
  set $KDC_FAMILIES
  Port=$(echo $2 | awk -F: '{print $2}')
  Comment Host=$CMShost Port=$Port

# 3. Verify Products installed
  for pr  in $Prods
  do  Product $pr
      cd $CANDLEHOME
      echo "\nOMA for $Prods \c"
      if [ -d $CANDLEARCH/$pr ]
      then  echo installed
            IVP_prod $pr
      else  echo not installed
      fi
  done

# CMS connectivity tests.  This relies on agents started by
# the Server() function and described in the $Agent array.  Max 1/product
  echo "\nCMS connectivity and seeding tests"
  Comment Host=$CMShost Port=$Port # Repeat connect info

# 17. Ping the CMS host.  "ping name 64 1" is the only consistent way
# to guarantee only one ping attempt on "almost" all Unix platforms.
# Note, HP and Sinix write brief description of ping failures to stdout,
#       AIX and Solaris write to stderr.
#       But all systems write excessively verbose output on ping success.

# 24593  Linux ping is slightly different from other Unix systems.
  if [[ $CANDLEARCH = li* || $CANDLEARCH = ls* ]]
  then     
    if ping=`ping -s 64 -c 1 $CMShost 2>&1`         # test ping return code.
    then Comment Host $CMShost pings OK
    else Complain 2 $ping
    fi
  else
    if ping=`ping $CMShost 64 1 2>&1`         # test ping return code.
    then Comment Host $CMShost pings OK
    else Complain 2 $ping
    fi
  fi

# 18 & 19. Analyze the agent results.
  Agent_stop
  Product="Distributed Databases" Severity
}

IVP_prod   () {
  typeset This PR=$1
  Sev=9   # means OK.  Low severity = bad

  # Push agent such that $Agent ( really ${Agent[0]} ) is null.
    set -A Agent '' "${Agent[@]}"

# 4.  kddos permissions
  This=kddos
  if  kddos auth >$Null 2>&1
  then Comment permissions OK for user $LOGNAME
  else Complain 3 permissions not set.
  fi

# 5. config files present for this host.
  cd $CANDLECFG

  This=$Product
  # D25589/D26807 - when multiple instances are defined, thus $HOST\_$PR\_*.cfg
  #          returns more than one file, linux displays an error , so determine
  #          whether at least one file matches that pattern in a slightly different way
  # if test ! -f $HOST\_$PR\_*.cfg
  LS=`ls $HOST\_$PR\_*.cfg`
  LS_RC=$?
  if [ "$LS_RC" != "0" ]
  then Complain 2 "No $Product config files apply to host $HOST"
       Comment    "Run CandleDBconfig $PR to configure.\n"
       return 1        # can't go any futher
  fi

# 6. config files valid and complete
  Prep_work       # Product-specifig, but not server-specific files.

  cd $CANDLECFG
  echo $Product Servers configured for $HOST:
  for file in $HOST\_$PR\_*.cfg
  do  this=${file#*${PR}_}
      This="  ${this%.cfg}"
      # source file in subshell, so that if it dies for a syntax
      # error, this shell stays OK.
      # Code note: next if tests ". $file" return code, not equality
      if   config=$(. ./$file && echo $db_installstatus)
      then Comment config status is $config

         if [[ $config = CONFIGSUCCESS ]]
         then . ./$file  # Source file here
            # Call Agent  to test CMS connection
              [[ -z $Agent ]] && Agent_start

            # Call IVP_Server to test server status & connectivity
              IVP_Server $file
         fi
      else Complain 2 "Config file $file is unusable.  " \
                      "Delete it and rerun CandleDBConfig"
      fi
  done
}

IVP_Server () {  # configfile must have already been sourced
  typeset status msg loc
  set -A Sev $Sev ${Sev[*]}     # Push severity, new Sev[0]=old Sev[0]

# 7.  Test for presence, readability of database files.
  Check read $db_home $db_initfilename

# 10. Test for database up by trying to log in an invalid id
  case $PR in
    (or)
      msg=$($db_home/bin/sqlplus bad/bad <$Null 2>&1 | fgrep ORA-)
       case $msg in
          (*ORA-01017*) status=up
       ;; (*ORA-01034*) status=down
       ;; (*ORA-01092*) status=down
       ;; (*ORA-01033*) status="in startup or shutdown"
       ;; (*ORA-01035*) status="up in restricted mode"
       ;; (*)           status=${msg:-down}
       esac
  ;; (oy)
       msg=$($SYBASE_OCS/bin/isql -Uinvalid -Pinvalid | tail -1)
#echo isql="\"$msg\""
#read # defect 17161
       case $msg in
          (*"Login incorrect"*)         status=up
       # Following line was added for Sybase 11.5 IMR 623684
       ;; (*"attempt to connect to the server failed"*)  status=up
       ;; (*"Server name not found"*)   status="an invalid server name"
       # Following line was added for Sybase 11.5 IMR 623684
       ;; (*"Requested server name not found"*) status="an invalid server name"
       ;; (*"Attempting to obtain a localized error message failed"*) status=up
       ;; (*"SQL Server is unavailable"*) status=down
       # Following line was added for Sybase 11.5 IMR 623684
       ;; (*"call to connect two endpoints failed"*) status=down
       ;; (*) status=$msg
       esac

       # 8.  Test for Sybase ctlib product installed.
       # We really require only ctlib.loc and cslib.loc
       loc=$SYBASE/locales/${SybLocale:=us_english/roman8}
       if [[ -f $loc/ctlib.loc && -f $loc/cslib.loc ]]
       then Comment "Open Client files OK"
       else Complain 2 'Open Client (CTLIB) not installed.'
       fi
  ;; (ox)
       set -- $(PATH=$db_home/bin:$PATH koxstat status 2>$Null)  # may be null
       eval status=$5             # eval strips "" from "on-line"
       case $status in
          (On-Line)   status=up
       ;; (Read-Only) status=up
                      Comment in read-only mode
       esac

     # 16. Check that onconfig file has ALARMPROGRAM set.
       typeset re='^[^#]*ALARMPROGRAM'          # RE for uncommented ALARM...
       set -- $(grep -s "$re" \
                ${db_initfilename:-$INFORMIXDIR/etc/onconfig.$db_sid})
       typeset it='onconfig ALARMPROGRAM'
       case $(basename $2 2>&-) in
            (koxerr)    Comment "$it is koxerr, only"
       ;;   (koxerr.sh) Comment "$it is koxerr.sh script, which calls"
              eval $(grep "$re" $2)  # execute the user line
              echo "      koxerr and ${ALARMPROGRAM:?}"
       ;;   (*)  Complain 3 "$it does not include koxerr"
       esac
  ;; (*)  Complain 1 "Invalid pr=$PR"
  esac

  Comment Database server is $status

# 11. Have a collector or somebody connect to the database

# Note, because this must run non-interactively against an existing
#       installation, it can't ask the user for a password.  We don't want a
#       command-line method to convert an encrypted password to plain text, so
#       we must use existing executables to connect:
#
#    to Oracle use korcoll with special collector script from Prep_work
#    to Sybase use koysql
#    to Informix use Informix's dbaccess program.

  if [[ $status != up ]]
  then Complain 1 cannot continue verification
       Severity
       return
  fi

  case $PR in
    (or)
       export COLL_DBVERSION=$db_ver
       # D25589 Added seting login and password env variables
       export COLL_LOGONID=$db_login
       export COLL_PASSWORD=$db_password

       cd $IVP

       # 12,13. Run an Oracle collector to check for grants, V$LATCH
       typeset sid=$db_sid      # for brevity
       rm $sid*                 # clean out any old results

       # Next statement uses "a=b cmd" to mean (export a=b; exec cmd)
       # This allows me to seed the collector's environment without
       # changing my environment.

       if [[ $COLL_DBVERSION = 8* ]]
       then Collector=ko8coll
       elif [[ $COLL_DBVERSION = 9* ]]
       then Collector=ko9coll
       else
            Collector=korcoll
       fi

       export COLL_VERSION=$db_type
       case $CANDLEARCH in
            (hp*)  Libvar=SHLIB_PATH
         ;; (sol*) Libvar=LD_LIBRARY_PATH
         ;; (aix*) Libvar=LIBPATH
         ;; (li*)  Libvar=LD_LIBRARY_PATH
         ;; (ls*)  Libvar=LD_LIBRARY_PATH
         ;; (lp*)  Libvar=LD_LIBRARY_PATH 
         ;; (lx*)  Libvar=LD_LIBRARY_PATH 
       esac
      
      #APAR IZ18824
      
      ARCHBIT=`echo $CANDLEARCH | sed 's/^.*\(.\)$/\1/'`
      if [ "$ARCHBIT" = "3" ] ; then
          dblib=$db_home/lib32:$db_home/lib:$db_home/lib64
      else
          dblib=$db_home/lib:$db_home/lib64:$db_home/lib32
      fi
            
      eval $Libvar=${dblib}:\$$Libvar

       PATH=$db_home/bin:$PATH   \
         COLL_SERVERID=$sid    \
         COLL_HOSTNAME=$HOST    \
         COLL_VERSION=k$PR      \
         COLL_PREFIX=$IVP       \
          $Collector -ddddd >$Null 2>&1    # running synchronously, w/debug
       # and 5 d's to catch all UPX0100 cursor dumps

   # 12, 13.  Analyze Oracle collector output:

   # Read the col.out file, and look for data-dump lines created by
   # the ko?coll -d option.  (For simplicity, I ignore continuations)
   # Write the text data into files named $sid.$cursor.  See the
   # Prep_work() function below for a list of cursors.

   # Note, because of an internal collector limitation, the data-dump
   # messages translate all special characters to '.', so a value like
   # V_$INSTANCE becomes V..INSTANCE, etc.

   # Awk program translation:
   # cursor name = 9th blank-delimited word from CGN1520 messages.
   # Select only UPX0100T (data-dump) messages with contination
   # indicator of "000:".  Extract 2nd asteric-delimited "word", and
   # write to a file named $sid.$cursor.  Note, using awk's option
   # to pre-set an awk varaiable from the command line to pass $sid in.

       # grouch: AIX awk doesn't honor dynamic changing of FS, use nawk
       # grouch: HP doesn't have nawk, thus using $nawk
       # grouch: Solaris awk honors   {print $2 > sid "." cursor}, but
       #                nawk requires {print $2 > (sid "." cursor) }

     $nawk '/^CGN1520/              {FS=" "; file=sid"."$NF }
            /^RSC0220W.*ORA-00942/  {FS=" "; print $4 >file }
            /^UPX0100T.*000:/       {FS="*"; print $2 >file }
            /^CAT1609/              {FS=" "; print $8 >file }
           '  sid=$sid ${HOST}_or_${sid}_col.out

       if [[ -s $sid.one ]]       # non-zero size of SELECT DUMMY FROM DUAL
       then Comment Login/password OK
         if [[ $db_login = sys ]]
         then Comment 'Connecting as sys.  No view/synonym/V$LATCH test.'
         else # we gotta check out views 'n syns 'n such.

           Discuss () {    # Discuss V_$ views,   Discuss V$ synonyms
             if grep -sl ORA-00942 $sid.$2 >$Null
             then typeset -u WHAT=$2
                  Complain 1 "Oracle user $db_login cannot read the SYS.DBA_$WHAT table,"
                  Comment    "cannot verify $*"
             # Compare names only (ignore leading V_$ or V$)
             # Translation:  cut off leading V_$ from $sid.views,
             #  change . to _ to undo collector changing all special
             #      chars to '.' ( so that SESS.IO becomes SESS_IO),
             #  sort,
             #  compare to requirement file (ignore case, trail blank)
             #  missing=any name in requirement file not in $sid.views.
             #  (note grep return=0 if some missing)
             else
             if missing=`cut -c$((1+${#1}))- $sid.$2 | sed 's/\./_/g' \
                | sort | diff -bi - korsyn.req | grep '>'`
             then Complain 2 "Required $1 $2 missing\n$missing"
             else Comment    "Required $1 $2 present"
             fi
             fi
           }
# D25589 - removed checking access permissions
#           Discuss V_$ views
#           Discuss V$  synonyms

           # Non-empty $sid.latch file means can't read V$LATCH.
           [[ -s $sid.latch ]] \
              || Complain 3 "Latch patch required.  See README."
         fi
       else Complain 2 connection failed
       fi

       cd $OLDPWD

  ;; (oy)
# Export COLL_SERVERID,COLL_LOGONID,COLL_PASSWORD to be used 
# by koysql ... Added for IMR628489 
      
       export COLL_SERVERID=$db_sid
       export COLL_LOGONID=$db_login
       export COLL_PASSWORD=$db_password
       export COLL_DBVERSION=$db_ver

       # set the collect name based on the database version.   
       if [[ $COLL_DBVERSION = 11* ]]
       then 
            Collector=koy12col
            koysql=koy12sql
       elif [[ $COLL_DBVERSION = 12* ]]
       then 
            Collector=koy12col
            koysql=koy12sql
       else
            Collector=koy15col
            koysql=koy15sql
       fi
       
       if [[ $CANDLEARCH = sol* ]]
       then
              LD_LIBRARY_PATH=$SYBASE_OCS/lib:$LD_LIBRARY_PATH
       fi
       if [[ $CANDLEARCH = aix* ]]
       then
              LIBPATH=$SYBASE_OCS/lib:$LIBPATH
       fi 
       # D27121: Specify the LD_LIBRARY_PATH      
       if [[ $CANDLEARCH = li* ||  $CANDLEARCH = lx* || $CANDLEARCH = lp* ]]
       then
              LD_LIBRARY_PATH=$SYBASE_OCS/lib:$LD_LIBRARY_PATH
       fi

       #  Try to execute Sybase stored procedure "sp_helpdb tempdb"
           if ans=$($koysql 1 " " sp_helpdb tempdb 2>&1)
           then Comment "logon successful"
              # 14.  TempDBsize analysis.
              # Known problem: next statements fail if $ans is a Sybase
              # error message explaining that tempdb is corrupted.
              # Only seen it happen once, doubt it will happen in field.

                set $ans
                if [[ $1 != tempdb ]]
                then # Sybase returned a nasty error upon connect
                    Complain 1 ${ans%tempdb*}
                    ans=tempdb\ ${ans#*tempdb}   # Chop off bad stuff
                fi

              # 15. Sybase sa_role test
                if $koysql 3 master 'select show_role()' | grep sa_role >$Null
                then Comment $db_login grants ok
                else Complain 2 koygrant.sql has not been run
                fi
           else Complain 2 \
             "connect failed, probable cause is Login/password"
             # Need sa connection to tell whether problem is in
             # our config file or no one ever "sp_addlogin candle"
           fi

  ;; (ox)
       # Try to connect to informix.
       if $INFORMIXDIR/bin/dbaccess sysmaster - <$Null >$Null 2>&1
       then Comment Unix user $LOGNAME has informix connect authority
       else Complain 2 Unix user $LOGNAME cannot connect to informix \
          "\n     Run koxgrant.sql for $LOGNAME"
       fi
       # need to audit sys tables. (Not implemeted.)
       # Next stmt:  change all koxcoll.ctl blanks to newlines,
       # remove all , or ; select words beginning with sys (any case).
       # note that broken line is an escaped newline in the new part of
       # s/old/new/g.
       true comments out next statement .... \
       sed 's/ /\
/g; s/[,;]//g' $CANDLEMISC/koxcoll.ctl | grep -i ^sys | sort -u
  ;; (*) Complain 1 "pr=$PR"
  esac

  Severity
}

# start or re-start agent.  If restart, Agent[$1] contains info
Agent_start () {

   typeset agent log
     agent=${db_type}agent
     Comment "${1+re-}starting $agent for CMS seed test"
     rm $IVP/$agent[:.]*        # Cleanup first
     #  Determine which IRA to trace.  The best solution would be
     #  to dynamically read the ${db_type}name.ctl file to find the
     #  enterprise table.  But, for now just hard-coding:
     kor=09 koy=01 kox=x0 eval ira=\$$db_type

     # Note: Sourcing Ct-environ in sub-shell to not pollute my PATH
     #       IVPagent parm for cosmetic only.  No reliance on it.
     # This does NOT have the old CTIRA_HOSTNAME for oracle and sybase,
     # so it will NOT work for agents older than 1q98.

     unset COLL_PREFIX_LOCAL

     KBB_RAS1="(UNIT:$db_type$ira ALL)" \
     KBB_RAS1_LOG=$IVP/$agent.log      \
      COLL_HOSTNAME=${KDCB0_HOSTNAME:-$HOST}   \
      COLL_SERVERID=$agent  \
      CTIRA_LOG_PATH=$IVP/  \
      COLL_PREFIX=$IVP      \
      KDC_FAMLIES=ip       \
        $agent IVPagent $MyPID >$IVP/$agent.log 2>&1 &

     # Record agent info for Agent_stop()
     # Agent[n] = agent name start-time, pid, LG0 filename, logfile
     #       n = passed idx if restart, else 0
     # Note assumption that servername unique in installation
     Agent[${1:-0}]="$agent $(Now) $! KDDIVP $PR"
}

# Evaluate results in LG0 and log files, stop agent when conclusive
Agent_stop () {

  typeset -i age wait a
  typeset This
  for age  in 0 30 60 120 180 300 -1
  do # Loop backwards through Agent[n], so do oldest agents first.
     a=${#Agent[*]}

     while ((1+ (a-=1) ))                   # stop when a=-1
     do [[ -z ${Agent[a]} ]] && continue
      # $1=koragent $2=start-time, $3=PID, $4=sid $5=pr
        set ${Agent[a]}
        typeset -L3 -u KXX=$1     # Ljust 3, upper: KOR, KOY, KOX
        Product $5
        This="CMS/$Product"

        if (( 0 < (wait=age-$(Now)+$2) ))
        then
          Comment "Waiting $wait seconds until $1 has run $age seconds"
          [[ $age = 30 ]] && Comment "Press Ctl-C anytime to interrupt test"
          sleep $wait
        fi

      # 18 & 19.  Analyze the LG0 file.

      Sev=2   # unless changed below
      LG0=$IVP/$1:*.LG0
      if [ -f $LG0 ]
      then
      # 24593  Linux subprocess cannot change parent's shell variables.
        cms=` 
          cut -c17- $LG0 | ( while read msgid message 
          # Code note: must escape blanks, quoting does not work;        
          do 
           case $message in
              (New\ log\ file\ created*)      cms=down
           ;; (CMS\ lookup\ failed*)          cms=down
           ;; (CMS\ located*)                 cms=inactive
           ;; (Starting\ HEARTBEAT*RNODESTS*) cms=unseeded
           ;; (Starting\ $Product*) cms="seeded but not recycled"
                                    # but keep reading...
           ;;  esac

          done # with reading LG0 file for now
          echo $cms)`
      else cms="to want us to wait"    # Agent still initializing...
      fi

      # If CMS seeded, but not sure it's recycled, check whether
      # UADVISOR has made a data request.  (Was koxx0agt.C on SINIX)
      if [[ $cms = seeded* ]] && fgrep agt.cpp $IVP/$1.log >$Null
      then Sev=9
           cms="properly seeded"
      fi

      if [[ $Sev = 9 || $age -lt 0 ]]  # done?
      then Agent[a]=''                 # done with this agent
           [[ $cms = down ]] && cms="down or not findable"  # so there
           Comment CMS is $cms                 # Sev already set
           Agent_kill $*

      elif [[ $cms = down && $age -gt 0 ]]  # not first or last pass
      then
         Comment "CMS was down when $1 started, restarting $1"
       # kill old agent, start a new one.
         Agent_kill $*
         typeset started=$2     # must preserve original start time
         # Call Agent_start, with db_type, db_sid and prd pre-set.
         db_type=${1%agent} db_sid=$4 prd=$5 Agent_start $a  # restart
       # Repair Agent[a] to restore original start time
         set ${Agent[a]}     # as modified by Agent_start $a
         Agent[a]="$1 $started $3 $4 $5"
      else Comment "CMS seems $cms..."
      fi
     done   # with this agent type
  done  # with age loop
}

# Note, earlier code issued a kill -2 for all but HP-UX, but that did not
# work for shared-memory agents not attached to a collector.
Agent_kill() {
  Comment killing "$1 PID=$3"
  kill -9 $3 2>&-
}

Prep_work () { # Support for Server():  Write scripts into log dir
  cd $IVP
  case $PR in
    (or)
  cat >korcoll.ctl <<\!     # \! means no $ substitution on text
cursor one;
     interval 10;
      INSERT_SELECT SELECT DUMMY FROM DUAL;
      INSERT INTO ONE(DUMMY) VALUES(:DUMMY);
cursor views;
     interval 10;
      INSERT_SELECT SELECT VIEW_NAME FROM SYS.DBA_VIEWS
         WHERE VIEW_NAME LIKE 'V_$%';
      INSERT INTO VIEW(VIEW_NAME) VALUES(:VIEW_NAME);
cursor synonyms;
     interval 10;
      INSERT_SELECT SELECT SYNONYM_NAME FROM SYS.DBA_SYNONYMS
         WHERE SYNONYM_NAME LIKE 'V$%';
      INSERT INTO SYN(SYN_NAME) VALUES(:SYNONYM_NAME);
cursor latch;
     interval 10;
     INSERT_SELECT SELECT ADDR FROM V$LATCH;
      INSERT INTO LATCH(ADDR) VALUES(:ADDR);
;; stop collector now.
cursor suicide;
     interval 10;
     fetch_exit fetchx2;
     script ps -fp $$ | tail -1 | awk '{print $3}' | xargs kill;
     INSERT_SELECT SELECT GOODBYE FROM ABCXYZ;
     INSERT INTO suicide(note) VALUES(:GOODBYE);
end;
!
  cat >kordict.ctl <<\!           # Paser requires at least one entry
name GOODBYE;
type integer;
initial 0;
end;
!
   #  Link korproc.ctl with real one in install.  Link must be soft,
   #  because kddunit testing may have the real korproc on another disk
      rm korproc.ctl
      ln -s $CANDLEHOME/misc/korproc.ctl .

  # Extract list of syns/tables from collector script to korsyn.req
  # Translation of next unreadable mess:
  #  sed the collector script:
  #  (1) delete all comments (lined beginning with ;)
  #  (2) delete all lines that don't contain V$  ($ escaped)
  #  (3) delete the V$LETGO line (it's a deliberate error)
  #  (4) change all semicolons to nulls
  #  (5) change all strings of consecutive blanks to a newline char
  #      (note the to-string is a literal newline)
  # Then select only lines beginning with V$ (ignore case), and
  # sort without regard to upper/lower case, eliminating dups, and
  # chop off leading V$.

    sed '/^;/d;  /[Vv]\$/!d; /V\$LETGO/d; s/;//;  s/  */\
/g' $CANDLEMISC/korcoll.ctl | grep -i '^V\$'| sort -fu | cut -c3- \
    > korsyn.req
  esac
  cd $OLDPWD

}

# Comment/Complain/Severity:
# These work with two semi-global variables:  $This and $Sev.
# This=identifying text to preceed any comment/complaint.
# Sev=Severity   (see function for meanings).
#   Sev[0] is severity of current test,
#   Sev[n] is a lifo stack of severities for higher contexts.
#
# Comment text           # Writes $This:  text
# Complain n text        # Writes $This:  text, Sev=max(Sev, n)

Severity() {
  Comment "OMA for $Prods \c"
  case $Sev in
       (2) echo will not work
    ;; (1) echo unverifiable
    ;; (3) echo warnings.
    ;; (9) echo verify OK
    esac
  echo
  set -A Sev ${Sev[1]}  # pop Sev
}

Complain() {
  [[ $Sev -gt $1 ]] && Sev=$1
  shift
  echo "$This:  $*"; }

Comment() { echo "$This:  $*" ; }

Now () { # seconds since midnight
  set `date "+%H %M %S"`; echo $(($1*3600 + $2*60 +$3)); }

Fatal() { echo "$*"; exit 1; }

Check () {    # Usage:  Check read/write obj obj...
  typeset obj action=$1
  typeset -lL1 rw=$1        # lower-case, 1 char
  shift
  for obj
  do  test -$rw $obj || Fatal user needs $action authority to $PWD/$obj
  done
}

# maint note, keep next line in parallel with Product_name() in kddfind.sh
Product   () {
  Product=$(sed  "/^$1|/!d; s/^.*|//; s/ Mon.*//" $CANDLEHOME/registry/proddsc.tbl)
}

# -----------------------------------------------------------------
# Main test
# -----------------------------------------------------------------
# if interrupted, kill any leftover agents
trap 'echo Interrupt received, killing agents
      ps -fu $LOGNAME | while read uid pid ppic c stime tty time cmd
              do if [[ $cmd = *IVPagent\ $MyPID* ]]
                 then echo killing ${cmd%% *}, PID=$pid
                      kill -9 $pid
                 fi
              done
              exit
     ' INT

# SybLocale=
typeset -i Sev=9                 # 9=No error

# Parse keyword args:
for Arg
do  typeset -l arg=$Arg
    case $arg in
       (o[ryx])    Prods=$arg
    ;; (*)         Servers=$arg\ $Servers
    esac
done

# Establish default values
  true ${Servers:=*} ${Prods:=or oy ox}
IVP_go

# $unix$ eclipse:tst/aix/bin/
