#!/bin/ksh
######################################################################
#                                                                    #
#        Licensed Materials - Property of IBM                        #
#        5724-B96                                                    #
#                                                                    #
#        Copyright IBM Corp. 2005 All Rights Reserved                #
#        US Government Users Restricted Rights - Use, duplication or #
#        disclosure restricted by GSA ADP Schedule Contract with     #
#        IBM Corp.                                                   #
#                                                                    #
######################################################################
######################################################################
#                                                                    #
#             (C) COPYRIGHT Candle Corporation 1996                  #
#                      All Rights Reserved                           #
#                                                                    #
#        Licensed Materials - Property of Candle Corporation         #
#         Duplication or disclosure prohibited without the           #
#        expressed written consent of the Candle Corporation.        #
#                                                                    #
#                                                                    #
######################################################################
# Usage: to be called by CandleDBconfig, only.
# Meaningful comments:
# @(#) $Header: /ctcvs/hclroot/src/kdd/kddfind.sh,v 1.4 2004/04/13 18:50:52 smyun Exp $
#
# Env Vars from CandleDBconfig:
#
# CANDLECFG = Config dir to write config files.
# CANDLETBL = Tables directory
# HOST      = Name of this host
# Debug     = -d if so requested on CandleDBconfig command line
#-------------------------------------------------------------------------------
# This is a set of functions invoked by CandleDBconfig.
# Only these functions have external visibility to CandleDBconfig:
#    Version       Return kddfind.sh version for logging
#    Verify        Read/write each config file to make all current
#    Product_menu  Configure products.
#    Batch         Pass parms for non-interactive execution
#    Read          Get user's answer to prompt, but don't log passwords
#
# user-modifable aliases to cover environmental differences
# alias grep=grep
# alias egrep=egrep
# alias awk=awk
#-------------------------------------------------------------------------------
# Change log
#-------------------------------------------------------------------------------
#   38215 - jlw - 05/07/07 - The Oracle discover task fails on SLES 10.  The 
#                            contents of file are cleared in routine PathNorm().
#   38215 - taf - 05/22/07 - There were 2 primary issues on SLES10
#                            1) variables defined with "typeset" are not 
#                               initialized by default, so we need to initialize
#                               them 
#                            2) echo is not interpretting the "\n" as a new line,
#                               so for all linux platforms we'll use echo -e to 
#                               ensure \n is interpretted as a new line
#-------------------------------------------------------------------------------


case $(uname) in (AIX) AIX=1;; (HP-UX) HPUX=1;; (SunOS) Solaris=1;; (Linux) Linux=1;; esac


if [ $Solaris ]
then 
     oratab=/var/opt/oracle/oratab    
else
     oratab=/etc/oratab             

fi

# D36738: Sybase executables on Linux/PPC64 require the IBM VAC libs, which are installed
#     in /opt/ibmcmp/lib64.  We need to add that directory to our LD_LIBRARY_PATH so 
#     that any attempts to run Sybase executables, e.g. dataserver in DBver(), will
#     be successful.
if [[ $Linux = 1 && `uname -m` = 'ppc64' ]]; then
    LD_LIBRARY_PATH="$LD_LIBRARY_PATH":/opt/ibmcmp/lib64
fi

Null=/dev/null
NL=${IFS#??}     # New-line char (third and last char of $IFS)

alias -x rm="rm 2>&-"

# 24593 "more" is different on Linux, -e is not supported.
if [[ $Linux = 1 ]]
then
  alias  more="more -d 2>&3 >$(tty)"  # allow more to work despite pipe
else
  alias  more="more -ed 2>&3 >$(tty)"  # allow more to work despite pipe
fi

######################################################################
# FUNCTIONS
######################################################################

Select_product() {
     echo "CandleDBconfig for $HOST $CANDLEARCH" >>$Logfile

   # Determine what products are installed.

   # CIDEV:  Next line assumes the current directory structure and
   #         product naming conventions.


   # Explicitly piping output to $Logfile only hides it from the user.
   # Everything the user sees is also written to $Logfile.

     Version >>$Logfile
     [[ $Verify = no ]] || Verify     # read & re-write config files

   # Did caller pass an explicit parm for database type?
   Product_menu $*
   #  Batch "$@"   # disabled
}

Read() {
   # Read prompt
   # Also echo results to logfile, but hide passwords
   typeset ans
   read ans?"$1"
   echo $ans
   [[ $1 = *password* ]] && ans=xxxx      # except passwords
   echo $ans>>$Logfile
}

Version() {
   # Write CTVC Header info to stdout
   echo '$Header: /ctcvs/hclroot/src/kdd/kddfind.sh,v 1.4 2004/04/13 18:50:52 smyun Exp $'
}


# Read/rewrite each config file that is older than the kddfind.sh
# script to bring them up to current format.

Verify() {

  cd $CANDLECFG
  # D38215 - initialized variables
  typeset -lL3 Prd=""     # Product calling requirement
  typeset kddfind=$(whence kddfind.sh) bad="" new="" host=""

  # 1st, rename any existing config files to Leonardo naming convention

    for file in *.@(ora|syb|ko?)
    do   [[ -r $file ]] || continue
         case ${file#*.} in (ora) pr=or;; (syb) pr=oy;; (kor) pr=ox;; esac
         eval $(grep db_hostname= $file)
         set -x
           mv $file $db_hostname\_$pr\_${file%.*}.cfg
         set +x
    done

  # Handle file only if older than this kddfind script
  for file in *_o[ryx]_*.cfg
  do  # Handle file only if older than this kddfind script
      [[ ! -r $file || $file -nt $kddfind ]] && continue

      if   (. ./$file)    # Verify integrity by executing in a sub-shell
      then [[ $file -nt $kddfind ]] && continue
           echo Verifing integrity:  $file
#           set $(IFS=_; echo $file)
#           Product_setup $2
#           Force=yes Update $file
      else bad=$bad\ $file
           echo "\
Warning, $file is not a valid ksh script and CANNOT be used as a
         monitoring agent configuration file."
      fi
  done

  if [ $bad ]
  then echo "
Unless you interrupt NOW, these files will be renamed to *.bad and ignored,
and CandleDBconfig will re-discover these databases and build new config
files for them.  Press Enter to continue, or interrupt and repair them."
       read
       for file in $bad
       do  mv $file ${file%.*}.bad
       done
  fi
}

Product_menu() {
  clear
  MenuHeader Welcome to CandleDBconfig
  cat <<\!
CandleDBconfig prepares the environment to run the IBM Tivoli Monitoring
for Databases monitoring agent.  CandleDBconfig:

1. Prepares a configuration file for each monitored server, so that the 
   monitoring agent knows how to talk to the database.

2. Verifies that the installation is correct and operational.

CandleDBconfig finds running databases much more easily and reliably than
non-running databases.  If you want to monitor databases that are not currently
running, you may wish to exit CandleDBConfig and start them.

Also, CandleDBconfig works best if you have more than 25 display lines.  Both
X-Windows and NT Telnet allow you to expand the number of lines without
disconnecting from Unix.  You don't have to issue any Unix commands to take
advantage of the increased lines.

!
kddos auth || echo "\
Your kddos root permission is not set.  This will impede database self-
discovery.  Please exit and have the super-user issue the following commands:

   cd $CANDLEHOME
   chown root */bin/kddos
   chmod u+sx */bin/kddos"

  typeset ans
  read ans?'Press enter to continue, or type "exit" to exit  '
  [[ $ans = exit ]] && exit

  while true
  do
  MenuHeader Select Search limits
       cat <<!
CandleDBconfig automatically searches for installed databases.  The search is
optimized to find running databases, but it can often find non-running ones.

Please select one of the following:

     1) Confine automatic search to running databases
     2) Also search for non-running databases  (slower)
     3) No automatic search.  Manual update only.
     4) Tutorial on automatic search techniques and limitations.

!
     MenuFooter Function menu
     if   ans=$(Select 1 2 0 "Help Scan")
     then case $ans in
              ( [012] ) Scan=$ans break
          ;;  (*) $ans
          esac
     fi
  done

   # if >1 product, show a product-selection menu.  Args are product abbreviations.
   # 99R1 change to CandleDBConfig forces $# always 1

  if (( $# > 1 ))
  then
     while true
     do MenuHeader Select Product
       typeset -i i=0
       for PR
       do  ProductDef="IBM Tivoli Monitoring for Databases: `Product_name $PR`"
           echo "               $((i+=1))) $ProductDef..."
       done
           echo "               $((i+=1))) Verify entire installation"
           echo "               $((i+=1))) List manual configuration steps..."
           echo "               $((i+=1))) Help for CandleDBconfig..."
       MenuFooter Main Menu
       if ans=`Select $* IVP "Help system" "Help intro"`
          then case $ans in
               (return | IVP ) $ans
            ;; (Help*)  # don't know which product, take defaults
                    Prod=any Prd=any Product=database Get_login=yes \
                    DoneMsg="All database servers found" $ans
            ;; (*)       Do_product $ans
            ;; esac
       fi
     done
  else Do_product $1
  fi
}

IVP() {
  # Propogate interrupts to IVP program
    trap true 2         # Ignore Ctl-C here
    clear
    kddivp.sh $*        # Note runs in different process
    trap 2              # Restore normal Ctl-C
}

# Select description, return only after the "Monitor - "
Product_name() {
  # D38638 fixed the product name 
  sed  "/^$1|Monitoring Agent for/!d; s/^$1|Monitoring Agent for //;s/|.*//; q" $CANDLEHOME/registry/proddsc.tbl
}

# code note: $Prd exists mostly for compatibility with kddos.  Plan to eliminate

Product_setup() {
  PR=$1
  Product=$(Product_name $PR) # full verbal product:  oracle

  case $PR in
     (or)  Prd=ora  Home_name=ORACLE_HOME  Init_name=init.ora    Get_login=yes
  ;; (oy)  Prd=syb  Home_name=SYBASE       Init_name=runserver   Get_login=yes
  ;; (ox)  Prd=inf  Home_name=INFORMIXDIR  Init_name=onconfig    Get_login=no
  ;; ( *)  Prd=     Home_name="DB home"    Init_name=init_file   # for generic help
  esac
}

# General menu coding:
# while true    # forever
#   MenuHeader  whatever     # Words used in kddhelp.sh to tell who called Help
#   cat <<!                  # Copy everything until ! in col 1 to stdout
#     1) This choice
#     2) That choice
# !
#   MenuFooter  whatever
#   if ans=`Select This That`
#   then $ans                # ans=return if user hit enter to exit
#   fi
# done

# Code note: if ans=`Select a b c` is NOT a comparison!!!  It means to assign
# results of Select() to ans, and also test Select's return code.  If the return
# code is 0, then we take action, which will be one of the passed parms, or
# "return" if he just hit enter).  If the return code is
# non-zero, the user typed an invalid response, so we just redisplay the menu.

Do_product() {
   # show menu for a product.  arg is product name: ora, syb, etc.
   Product_setup $1             # initialize product variables:

   if (( Scan ))
   then echo "Performing initial scan for $Product.  Please wait..."
        Scan
   else echo "Initial scan bypassed"
   fi

   # 1st menu item depends on whether we need login/passwords.
   # these vars also used by Locate.
     if [[ $Get_login = yes ]]
     then DoneMsg="All $Product servers found.  Ready to specify login/passwords..."
          DoneAct=Login
     else DoneMsg="All $Product servers found.  Configuration complete"
          DoneAct=return
     fi

   while true
   do  MenuHeader $Product Options
       cat <<!
     1) $DoneMsg
     2) Optional disk searches, manually add servers...
     3) Display or Modify defined servers...
     4) Exclude/Include a server from the configuration..
     5) Verify Monitoring Agent for $Product installation
     6) List external configuration steps for Monitoring Agent 
        for $Product...
     7) Help...
!
     MenuFooter Function menu
     if ans=`Select $DoneAct Locate Modify Exclude "IVP $PR" \
                    "Help system" Help`
     then $ans
     fi
   done
}

# Next code outside of functions so that the trap on exit occurs after all done
My_fuser=$CANDLETMP/fuser.$CANDLEARCH
trap '[[ -a $My_fuser ]] && rm -f $My_fuser' EXIT

Scan() {

  [[ $PR = or ]] && or_oratab="or in $oratab" || or_oratab=
  echo  Scanning servers currently running $or_oratab
  cd $CANDLECFG
  Inventory before

  # Find a copy of fuser that I can execute.
  # see fuser triva in comments at end of this file
  typeset fuser
  if   fuser=$(whence fuser)
  then true
  elif [[ -x $My_fuser ]]
  then fuser=$My_fuser
  elif fuser=/usr/sbin/fuser
       [[ -x $fuser ]]                      # (HP can't execute it)
  then true
  elif fuser=/sbin/fuser                    # D25135 - Linux RedHat
       [[ -x $fuser ]]                      
  then true
  else cp -f $fuser $My_fuser               # copy to my dir...
       fuser=$My_fuser                      # remember have done so
       chmod +x $fuser                      # w/ x-bit...
  fi

  # Make sure it works, else use kddos.
    for fuser in $fuser "kddos fuser" false
    do  $fuser . >$Null 2>&1 && break
    done

  # --------------------------------------------------------------------------
  # Pass 1: Scan_$PR return formats:  ?=maybe not there
  #   /home  pid sid? reason?
  #   ~owner pid sid?         (HP Sybase only: ~owner:partial/dbhome )
  # --------------------------------------------------------------------------

  # D38215 - initialized variables
  typeset Home="" home=""     # Home is modified only by Scan_check.  home=scratch pad.
  typeset pid="" sid="" ok="" Lost Serv
  typeset -i n          # n=common idx for Lost and Serv
   # F23012/23473 - changed this from piping to a while-loop
   # to storing in an env var and using a for-loop because of a bash issue
   ScanResults=`{  Scan_$PR
      eval [[ -n \$$Home_name ]] && eval echo \$$Home_name # this is free
   } | sort -u` 
   OLD_IFS=$IFS
   IFS="
"
   for entry in $ScanResults
   do let n+=1
      if   [[ $entry = ~* ]]              # ~owner?
      then Lost[n]=$n\ $entry
      elif home=$home\ ${entry%% *}       # always save /possible/dbhome
           [[ $entry != *\ * ]]           # only one word=bare home.
      then true                           # thats all for this line
      elif [[ $entry = /*\ [1-9]* ]]      # /home pid = a Serv entry.
      then Serv[n]=$n\ $entry
      else IFS=" "
           say Invalid Scan_$PR result: $entry
           exit 99
      fi
   done
   IFS=$OLD_IFS

  for break in action            # let "break" exit code block anytime
  do  Scan_check $home && break

  # --------------------------------------------------------------------------
  # Pass 2: Bring in home from existing config files, and see if that finds
  # everything.  Note, this relies upon the "quotes" in config values.
  # enh:  bring in sids, too
  # --------------------------------------------------------------------------

    files=$(ls $HOST\_$PR\_*.cfg 2>&- ) &&
       Scan_check $(fgrep -s db_home= $files | cut -f2 -d\" ) &&
         break

  # --------------------------------------------------------------------------
  # Pass 3: Resolve any pids with gcore, if available.
  # --------------------------------------------------------------------------

    if   [[ -a /usr/bin/gcore ]]    # may be -x only to root
    then for lost in "${Lost[@]}"
         do set   $lost; n=$1     # n home pid
            if    all3=$(kddos look $Prd $3)
            then  unset Lost[n]
                  Serv[n]="$n $all3 kddos_look"
            fi
         done
         Scan_check && break     # should always be complete now.
    fi

  # --------------------------------------------------------------------------
  # Pass 4:  search in $HOME directories of DBowners. (fast)
  # Weakness: .sh_history permissions are 600
  # --------------------------------------------------------------------------

  Scan_check $(
    # F23012/23473 - need to look at this further !!!!!!
    for lost in "${Lost[@]}"
    do  lost=${lost#* }       # delete leading "n "
        echo ${lost%%[ :]*}   # just ~owner
    done | sort -u | while read owner
    do eval Script $owner/.@(profile|login|cshrc|history|sh_history)
    done | sort -u) && break

  # --------------------------------------------------------------------------
  # Pass 5: disk search:  See comments outside function.
  # --------------------------------------------------------------------------
  
    typeset fs_users

    # create a list of filesystem-users (only once)

    true ${fs_users=$(
      # field=for AIX, df output lists filesys name last, all else first.
      #     c=fuser parm for "while filesys" (must be null for AIX)
      # fuser 2>&1 so we get the usage-letters after each pid.
      [ $AIX ] && field=NF  c= || field=1  c=-c
      # F23012/F23473 - Linux does not support the -c parm on fuser 
      [ $Linux ] && field=NF  c= 
      \df | awk "/\// {print \$$field}" | xargs $fuser $c 2>&1)}

    for lost in "${Lost[@]}"
    do  set $lost
        n=$1 owner=$2 pid=$3 sid=$4

        if [[ $owner = *:* ]]
        then  partial=${owner#*:} owner=${owner%:*}
        else  partial=
        fi

        # Find filesystem for text(pgm) file.  fuser result is $pid, immediately
        # followed by various (unpredictable letters.  I want t=text (executable) file

        filesys=$(echo "$fs_users" | grep " $pid[^ ]*t")   # req. leading blank
        filesys=${filesys%%:*}            # trim to filesystem name (must be only 1)

        # don't search same owner's home twice.
          if [[ $owners != *:$owner:* ]]
          then eval uhome=$owner          # expand ~owner to real path
              # if owner's $HOME on same filesystem as $filesys, look there first.
              same_fs $uhome && owners=:$owner:$owners || uhome=
          fi

        owner=${owner#~}      # because kddos wants it that way

        # Search hierarchy: partial(if any), uhome(if applicable), filesys, filsys(exhaustive)

        for here in $partial $uhome $filesys /$filesys
        do say  Beginning search under $here
           fs=1               # confine search to one filesys, unless...
           [[ $here = $partial ]] && { same_fs $partial || fs=; }
           dbhome=$(kddos find $Prd $here $owner $fs) || continue # if no hits

           # Oracle exception: ORACLE_HOME must be exactly as oracle expects,
           # another path to the same location is not good enough (IMR 522277)
           # Note, Oracle only supports ORACLE_HOMEs that are pure physical paths,
           #       but does not enforce this.  ORA80A41 is an counter-example.
           # kddos find only produces physical paths

           # techniques:
           # 1. (only works for oracle 7) look in the control$SID.ctl files (not imp)
           # 2. look for line in orainst/root.sh that says:
           #   ORACLE_HOME=${ORACLE_HOME:=/path...}
           # and execute it.s

           [[ $PR = or ]] &&
            dbhome=$(
              for home in $dbhome
              do  if   line=$(fgrep ORACLE_HOME:= $home/orainst/root.sh 2>&-)
                  then unset ORACLE_HOME           # force ${ORACLE_HOME:=...} to fire
                       eval $line
                       # do not accept ORACLE_HOME unless it really points to this home
                       [[ $home -ef $ORACLE_HOME ]] && echo $ORACLE_HOME || echo $home
                  else echo $home
                  fi
              done | sort -u)

           Scan_check $dbhome  && break 3   # (for here, while(n), for break)
           [[ -z ${Lost[n]} ]] && break
        done
    done     # for lost in $Lost
  done       # for break in action

  # d50644
  
  if [ $Scan = 2 ] && [ $PR = or ]
  then
      Find_nonrunning_$PR
  fi

  # --------------------------------------------------------------------------
  # Final report: servers that could not be found
  # --------------------------------------------------------------------------
    if (( ${#Lost[*]} ))
    then echo Following servers could not be found:
         for lost in "${Lost[@]}"
         do  echo ${lost#* }  # but not index
         done
    fi

  # Determine sids for running servers, and formally add them.
    typeset -i f
  # 1st, sort into dbhome order.
    # F23012/F23473 Need to fix this!!!
    if [ $Scan = 2 ] ; then
       scan_serv=`for serv in "${Serv[@]}"; do echo ${serv#* }; done`
    fi
    # echo "DEBUG: scan_serv = $scan_serv"

    { [[ $Scan = 2 ]] && echo $Home | xargs -n1
      for serv in "${Serv[@]}"; do echo ${serv#* }; done   # convert to row/col, no n
    } | sort -r |                          # foreach home, sort $Home lines after $Serv,
    while read home pid sid reason
    do
        if   [[ -n $sid && $sid != . ]]
        then echo $home $sid $reason
        elif [[ -n $pid && -n $Solaris ]] && ok=$(kddos look $Prd $pid)
        then set -- $ok
             echo $home $3 $reason,kddos_look
        elif #  Must search dbhomes for sids, but only when home changes.
             if [[ $home != $home1 ]]
             then  home1=$home                # dont do this home again
                   # convert finds=(home sid reason init :pids) to a self-indexed array.
                   unset finds
                   f=0
                   # F23012/F23473 need to fix this!!!!!
                   Find_sid_$PR $home | while read this
                   do let  f+=1
                      finds[f]="$f $this"
                   done
             fi
             # Scan=2 only: a home-only line means this dbhome does not have any
             # running servers.  So, we just send any remaining finds.
             [[ -z $pid ]] &&
                  (IFS=$NL; echo "${finds[*]}" ) | cut -s -d\  -f2- && continue

             # parse note: finds[n] contains "fake" fuser output, :pid\o pid\...
             hit=$(IFS=$NL; echo "${finds[*]}" | grep ":$pid[^ ]*o")

        then # hit=sid running this pid (2+ if mult initfiles pretend to use same DB)
             #F23012/23473 Need to fix this!!!
             echo "$hit" | while read f etc
             do    echo $etc
                   unset finds[f]
             done
        else say  "Could not determine server name for:\n" \
                  "    PID=$pid home=$home, ignored"
        fi
    # F23012/23473 Need to fix this!!!
    done | while read dbhome sid reason init etc
    do   [[ $dbhome != $home1 ]] && dbver=$(DBver) home1=$dbhome
         Update server=$sid dbhome=$dbhome initfile=$init dbver=$dbver \
                reason=$reason $Batch_parms
    done

  Inventory after
}

# Scan_check potential_dbhome...
# Updates Home=accumulated list of dbhomes, which is localized to Scan,
#              but only updated here.
#         Lost=servers with unknown homes,
#         Serv=found running servers.
# returns 0 if all servers found AND Scan=1  !!          (never called if Scan=0)
#         1+ if Scan=2, indicating do not stop search
#
# The logic of Scan_check assumes that items in Lost move from
# Lost[n] to Serv[n] only.  It keeps no record of fuser output from
# previous calls, only the $Home entries from previous calls.
# The $Home built by this process is also used for discovering
# non-running databases.

Scan_check() {
  # args are potential new dbhomes, with dups eliminated.
  # Keeps $Home=list of potential dbhomes already checked.  Only updated here.

  typeset -i OK          # OK=0 means scan done (req Scan==1)
  (( OK = ${#Lost[*]} || Scan==2 )) || return 0     # trival case: none lost
  (( $# )) || return $OK                            # No args, no action.

   # homes = fuser output of new dbhomes
   # xtr   = item under home that server uses
   # x     = how used:  c=current dir, t=text(executable)
   # D38215 - initialized variables
   typeset x="" xtr="" homes="" home=""

   case $PR in
       (or) x=c xtr=/dbs
    ;; (oy) x=t xtr=/bin/dataserver
    ;; (ox) x=t xtr=/bin/oninit
   esac

   # 1st, homes=args, purged of any unqualified ( ! -d $home),
   # any already in $Home, and any dups

     homes=$(
       for home
       do  [[ -d $home && $Home != *" $home "* ]] && echo $home   # note require blanks
       done | sort -u)
     [[ -z $homes ]] && return $OK         # none new this call

   # convert $homes to an list of processes using the $homes

   if [[ -z $xtr ]]
   then  homes=$($fuser 2>&1 $homes)
   else  homes=$(echo "$homes" | xargs -i $fuser 2>&1 {}$xtr)
   fi

  # Remove any found items from $Lost, add to $Serv (use same index)
  # D38215 - initialized variables
    typeset -i n
    typeset pid="" sid=""

    for lost in "${Lost[@]}"
    do set $lost              # n ~owner pid sid
       n=$1 pid=$3 sid=${4-.} # sid may be unknown
       if [[ $homes = *\ $pid*([! ])$x* ]]
       then  home=$(echo "$homes" | grep " $pid[^ ]*$x")
             Serv[n]="$n ${home%$xtr:*} $pid $sid fuser"
             unset Lost[n]
       fi
    done

  # Update $Home with new ones (with blanks around each dir)
    set -- $(echo "$home" | sed "s#$xtr:.*##") ''
    Home="$Home $* "

  return $(( ${#Lost[*]} || Scan==2 ))
}

# Initial scan functions:  Scan_$PR
#   no need to localize,  called only by sub-process.
#   returns rows of  home pid sid reason
#     home=/path            if determinable
#         =~owner           if not
#         =~owner:/partial  Sybase on HP only: only partial path known
#     pid
#     sid if known
#     reason = data source, if relevant.  1 word, no ':'

# ORACLE INITIAL SCAN
# I used to filter all non-aix servers through oratab, and only
# report the ones not found there.  But, oratab can be wrong,
# so it's safer to pass them back and let Scan_check match them.

Scan_or() {

  # Gather all possible information from the ps command.

  ps=$(ps -ef)                    # only do once

  echo "$ps" | grep ora_pmon | while read who pid rest
  do sid=${rest#*ora_pmon_}
     if [ $AIX ]                 # can get home from env string
     then typeset this=`ps eww $pid`   # env string
          this=${this#*ORACLE_HOME=}  # chop off leading junk
          where=${this%% *}          # chop off trailing junk
          reason=AIX_ps_ceww
     else where="~$who"        #d50644
          reason=
     fi
     echo "$where $pid $sid $reason"
  done

  # 593505
  # orasrv is the boot-startup server, which may have a path.  Send if so
  # code:  sed '\:string:cmd'  means use : as context address delimiter
  # sed:    only trailing /*/bin/orasrv; remove all before ' /'; remove /bin/orasrv

    echo "$ps" | sed '\: /.*/bin/orasrv$:!d; s:.* /:/:; s:/bin/orasrv$::'

  # Send everything in the oratab file

    grep -sv "^#" $oratab | cut -s -f2 -d:  # cut -s = suppress lines w/o :
}

# Sybase is an exception to normal rules:
# 1. The ps command usually has the full path and server name.
# 2. The server environmental variables can be meaningless.
#    (All server parms are usually in the runserver file)
# 3. The dataserver pwd is meaningless unless the executable is a relative path.
# 4. HPUX truncates "ps -ef" CMD  to 60 bytes, Solaris to 79.

Scan_oy() {

 typeset -i psn   # numbers below are empircal
 case $(uname) in (AIX) psn=999999;; (HP-UX) psn=60;; (SunOS) psn=79;; (Linux) psn=999999;; esac
 typeset -r pgm=/bin/dataserver
 # D38215 - initialized variables
 typeset sid="" cmd="" pid="" etc="" home=""

 # grouch :Solaris ps -e truncates executable names to 8 chars
 # F23012/23473 Need to fix this!!!
 ps -e | grep dataserv | grep -v grep | while read pid etc
 do  ps=$(ps -fp $pid)
     cmd=${ps##*:[0-9][0-9] }                       # cmd is after TIME(mm:ss)

     # Get Sybase home, if possible.

     if   [[ $cmd = /*$pgm* ]]             # complete path in cmd line?
     then home=${cmd%%$pgm*}               # remove trailing /bin/dataserver

     elif [ $Solaris ] &&                  # Solaris....
          kddos look syb $pid              # look under the covers (home, pid sid -> stdout)
     then continue                         # All done with this server!

     elif [ $AIX ] &&                      # AIX ps -f complete, so $cmd is a relative path
          env=$(ps ceww $pid) &&           # complete env string
          pwd=${env#*PWD=}    &&           # $PWD + blank + junk
          text=${pwd%% *}/${cmd%% *} &&    # $PWD/$cmd
          [[ -a $text ]]                   # be sure itsa file. (no guesswork, so no $fuser check)
     then cd ${text%$pgm}                  # cd to home
          home=$PWD                        # resolve any /./ or /../ in $text

     elif [[ $cmd =  /*/bin/* ]] &&        # HP: at least /bin/ here?
          home=${cmd%%/bin/*}   &&         # its a potential home...
          text=$home/bin/dataserver &&     # running this executable...
          [[ -a $text ]]            &&     # provided it exists...
          [[ $($fuser 2>&1 $text) = *\ $pid*([! ])t* ]]  # and we are really using it.
     then true

     elif set -- ${ps#*$NL}                   # owner=$1
          [[ $cmd = /* ]]                  # HP: is it a truncated path?
     then home=~$1:$cmd                    # cmd is an incomplete sybase home.

     else home=~$1                         # useless relative path, let Scan disk search.s
     fi

     # get server name, if possible

     if   [[ ${#cmd} -le $psn ]]           # complete cmd and args?
     then case $cmd in
		( *-s*-e* ) 
          		sid=${cmd#*-s}
               		sid=${sid%% *}
	;;	( *-s*-d* )		   # if the pattern is *-s*-d* on HP1020
			sid=${cmd#*-s}
			sid=${sid%% *}
	;;	( *-d*-s* ) 	   # if the pattern is *-d*-s* for HP1020
			sid=${cmd#*-s}
			sid=${sid%% *}
	;;	( * )                      # try to find the server name other ways - defect 12244
		if [ -f /usr/ucb/ps ]; then
			psresults=`/usr/ucb/ps -auxwww | grep dataserv | grep -v grep`
			for word in $psresults; do
				result=`echo "$word" | grep "^-s"`
				if [ -n "$result" ]; then
					sid=`echo "$word" | sed 's/-s//g'`
					break
				fi
			done
		fi
		if [ -z "$sid" ]; then
			curpwd=`pwd`
			cd "$home/install"
                        # F23012/23473 Need to fix this!!!
			ls RUN_* | while read name
			do
				result=`echo "$name" | sed 's/.*RUN_//g'`
				asid=`grep "\-s$sid" "$name" | sed 's/.*-s//g' | awk '{print $1}'`
				if [ "$asid" = "$result" ]; then
					sid="$asid"
					break
				fi
			done
			cd $curpwd
		fi
	;; esac

     elif [[ $cmd = *-s*\  ]]              # truncated args with complete -s
     then sid=${cmd#*-s}
          sid=${sid%% *}

     else sid=.                            # no clue
     fi

     echo "$home $pid $sid ps"
 done
}

Scan_ox() {
    # F23012/23473 Need to fix this!!!
    ps -ef | grep oninit | while read who pid ppid etc
    do [[ $ppid = 1 ]] || continue        # only want top-level server
       if [ $AIX ]                        # can get home from env string
       then env=$(ps eww $pid)            # env string
            home=${env#*INFORMIXDIR=}     # chop off leading junk
             sid=${env#*INFORMIXSERVER=}  # chop off leading junk
            echo "${home%% *} $pid ${sid%% *}"
       else echo "~$who $pid"
       fi
    done
}

Find_nonrunning_or() {
    #echo "DEBUG: Searching for non-running servers"
    # 2 sets of files will be looked at to find ORACLE_HOMEs and SIDs
    # a) using the ORACLE_HOMEs found earlier, stored in ScanResults,
    #    find the spfiles in $ORACLE_HOME/dbs. The spfile filename contains
    #    the sid, the filename format is spfile<sid>.ora (by default)
    # b) using the ORACLE_HOMEs found earlier, stored in ScanResults,
    #    find the sids in $oratab.
    do_spfile_search="YES"
    do_oratab_search="YES"
    # --------------------------------------------------------------------------
    # Search spfiles
    # --------------------------------------------------------------------------  
    if [ "$do_spfile_search" = "YES" ];
    then
        #echo "DEBUG: 6a. ScanResults = $ScanResults"
        OLD_IFS=$IFS
        IFS="
"
        for line in $ScanResults 
        do 
            dir="$line"
            #echo "DEBUG: dir = <$dir>"
            # if the directory exists, then search it for spfile's
            if [ -d ${dir}/dbs ] ;
            then
                spfiles=`find ${dir}/dbs | grep "^${dir}/dbs/spfile.*\.ora$"`
                #echo "DEBUG: spfiles = <$spfiles>"
                for fqfilename in $spfiles
                do
                   sid=${fqfilename#${dir}/dbs/spfile} # remove the fq filename, thru spfile preceding the SID
                   sid=${sid%.ora}   # remove the .ora following the SID
                   #echo "DEBUG: sid = <$sid>"
                   index=0
                   while true
                   do 
                       # if we get to the end of Serv and haven't found a match, then add it and break
                       if [ "${Serv[index]}" = "" ] ;
                       then 
                           # if a cfg file already exists, then this is a previously known sid, don't add it
                           if [ ! -f $CANDLEHOME/config/$HOST\_$PR\_$sid.cfg ];
                           then
                               #echo "DEBUG: Adding Serv[$index]=$index $dir 1 $sid oratab"
                               Serv[index]="$index $dir 1 $sid spfile"
                           fi
                           break
                       fi
                       #echo "DEBUG: Serv[${index}] = ${Serv[index]}"
                       # the Serv[x] entries are space separated values, and spaces are allowed
                       # in file names, so whil this checking is not foolproof, it's the best we can
                       # do given the format of Serv
                       if [[ ${Serv[index]} = *${dir}* ]] && [[ ${Serv[index]} = *${sid}* ]] ; 
                       then
                           #echo "DEBUG: not adding $sid, it would be a duplicate" 
                           break
                       fi
                       let index+=1
                   done
                done
            fi
        done
        IFS=$OLD_IFS
    fi
    # --------------------------------------------------------------------------
    # get sid's from oratab
    # --------------------------------------------------------------------------  
    if [ "$do_oratab_search" = "YES" ];
    then
        #echo "DEBUG: 6b. ScanResults = $ScanResults"
        OLD_IFS=$IFS
        IFS="
"
        for line in $ScanResults 
        do 
            dir="$line"
            #echo "DEBUG: dir = <$dir>"
            # if the directory exists, then we'll assume it's a valid ORACLE_HOME
            if [ -d $dir ] ;
            then
                # get all lines in oratab that match this home dir, but 
                # don't start with an asterisk
                oratab_results=`grep "$dir" $oratab | grep -v "^\*"`
                for oratab_line in $oratab_results
                do
                    sid=`echo $oratab_line | cut -f1 -d:`
                    #echo "DEBUG: sid = <$sid>"
                    index=0
                    while true
                    do  # if we get to the end of Serv and haven't found a match, then add it and break
                        if [ "${Serv[index]}" = "" ] ;
                        then
                            # if a cfg file already exists, then this is a previously known sid, don't add it
                            if [ ! -f $CANDLEHOME/config/$HOST\_$PR\_$sid.cfg ];
                            then
                                #echo "DEBUG: Adding Serv[$index]=$index $dir 1 $sid oratab"
                                Serv[index]="$index $dir 1 $sid oratab"
                            fi
                            break
                        fi
                        #echo "DEBUG: Serv[${index}] = ${Serv[index]}"
                        # the Serv[x] entries are space separated values, and spaces are allowed
                        # in file names, so whil this checking is not foolproof, it's the best we can
                        # do given the format of Serv
                        if [[ ${Serv[index]} = *${dir}* ]] && [[ ${Serv[index]} = *${sid}* ]] ; 
                        then
                            #echo "DEBUG: not adding $dir / $sid, it would be a duplicate" 
                            break
                        fi
                        let index+=1
                    done
                done
            fi
        done
        IFS=$OLD_IFS
    fi
}


# Find_sid_$PR: locate all servers under a dbhome.
# Also called by Locate fns to get sids for
# non-running servers, but with fuser=echo to disable it.
# Return format:
#    dbhome sid reason initfile :processes....
#
# We put the colon in front of the processes so that the caller can grep for
# ":$pid" with no possibility of the pid accidentally matching an earlier
# part of the line.

# Find_sid_or will never be called for running servers, so it doesn't try to
# extract processes.  Serious cheat:  Find_sid_or returns only 1 row per sid
# found, no matter what the reason.  Also note that this mixes guaranteed good
# sid's extracted from initfiles (via db_name= in kddos) and questionable ones
# extracted from filenames.

Find_sid_or() {
  # possible: dbs/  config* ora|init!(*_0).ora (look in ifile)
  # confirm:  dbs/@(ctrl[1-3]*.ctl|@(temp|tool|usr|sgadef)*.dbf)
  # this needs work, no -a *.dbf test yet
  home=$1
  cd $1/dbs
  for file in *
  do  case $file in
         (@(init|config)!(*_0).ora )
            sid=$(kddos orasid $file) &&    # only if kddos $?=0.
               echo $home $sid kddos_orasid \
                    $( [[ $file = init* ]] && echo $file )
            continue                        # already written
      ;; (ctrl[1-9]*.ctl) sid=${file#ctrl[1-9]}
                          sid=${sid%.ctl}
      ;; ( @(temp|tool|usr|sgadef)*.dbf )
             sid=${file#@(temp|tool|usr|sgadef)}
             sid=${sid%.dbf}
      ;; (*) continue               # ignore all others
      esac
      echo $home $sid $file # reason=$file, initfile unknown
  done | sort -u +0 -2
}


Find_sid_oy() {
  home=$1
  cd $1/install
  for initfile in $(grep -ls bin/dataserver !(*errorlog*|*.log|sybinit|@(start|show)server))
  do  if [[ -r $initfile ]]
      then cmd=$(egrep -v "^[${IFS%?}]*#" $initfile)    # exclude comment lines (# after any whitespace)
      else cmd=$(kddos sybsid $initfile)
      fi
      # does the database exist? anyone using it?
        [[ $cmd = *-d* ]] || continue
        x=${cmd#*-d}; database=${x%% *}
        [[ -a $database ]] || continue

      if [[ $cmd = *-s* ]]               # determine sid from file
      then x=${cmd#*-s}; sid=${x%% *}
      else sid=SYBASE
      fi

      set -- $($fuser 2>&1 $database) && shift
      (($#)) && reason=fuser || reason=initfile    # why do we think this is valid?
      echo $home $sid $reason $PWD/$initfile :$*          # return list of sids using this dataset
  done | sort -u
}

# An informix onconfig file can be named anything, so we check
# all files in the etc directory which are less than 10K.
# Code notes:
# a) AIX find doesnt support -follow, so we don't follow symbolic links in AIX.
# b) "find -print | xargs" is far more efficient than "find -exec"
# c) We return the pid of the database users' parent, because because Scan_inf
#    returns the pid of only the parent process, and the parent itself does not
#    use the database.

Find_sid_ox() {
  home=$1
  cd $1/etc
  # F23012/23473 Need to fix this!!!
  find !(*.log|*.out) -prune -size -10000c \
        $(test $AIX || echo -follow) -print |
  xargs kddos infsid | while read initfile sid database
  do      set -- $($fuser 2>&1 $database);   shift
          if   (( $# ))
          then reason=fuser
               ps=$(ps -fp ${1%o})
               set ${ps#*$NL}           # ppid is $3
          else reason=initfile
          fi
          echo $home $sid $reason $PWD/$initfile :$3o   # pretend open
  done | sort -u
}

# arg in same filesystem as $filesys? (not work on AIX or HP9)
same_fs() { [[ $(df -n $1) = $(df -n $filesys) ]]; }

#---------- here ends automatic-scan functions --------------------------

Locate() {
   # fool "set $($fuser anything) && shift" into creating no args.
     typeset fuser=echo

   while true
   do MenuHeader $Product Disk Search
     typeset -l lproduct=$Product       # force to lower case

     cat <<!
     1) Search of scripts in the PATH, looking for $Home_name
     2) Display a list of Unix users that are possible $Product owners....
     3) Specify a search starting point, usually the \$HOME of the
        $Product owner.  For example, ~$lproduct.  You can select
        this option multiple times with different starting points.
     4) Manually add a server
     5) $DoneMsg
     6) Help
!
     MenuFooter Search Options
     if ans=`Select Path Userlist Search Add_manual $DoneAct Help`
     then $ans
          [[ $ans = $DoneAct ]] && return
     fi
   done
}

Path() {
  # Search path for Home_name
  Inventory before
  typeset path=${PATH#*:}      # exclude product bin directory
  while [[ -n $path ]]
  do   dir=${path%%:*}                # 1st entry
       # Remove all but 1st entry from path
         [[ $path = *:* ]] && path=${path#*:} || path=
       # Exemptions for performance
         [[ $dir = $CANDLEHOME/* ]]           # Exempt myself
         [[ $dir = . ]] && continue           # Exempt myself
         [[ $dir = ?(/usr)/?(s)bin ]] && continue # Exempt unix bins
         [[ $path = *:$dir:* ]] && continue   # Ignore most dups in path
     say  searching $dir
     # Cheat: AIX find doesn't support -follow, so ignore links
     # Note, no check for execute bit, because not needed to ". script"
     if [ $AIX ]
     then file $dir/*
     else file $(find $dir/* -prune -follow -type f -print 2>&-)
     # F23012/23473 Need to fix this!!!
     fi | while read this
     do #  Results of file is "file: description"
        #  reject scripts provided by Oracle, and
        #  for others, description must contain text or script.
        [[ $this = !(*/?(c)oraenv.*|*/dbhome):*@(text|script)* ]] &&
           Script ${this%%:*}
     done
  # F23012/23473 Need to fix this!!!
  done | sort -u | while read dbhome
  do [[ -d $dbhome ]] && Add_servers $dbhome
  done
  Inventory after
}


Script() { # return any dbhome assignment found in passed files
  # F23012/23473 Need to fix this!!!
  fgrep -s $Home_name $* | while read line
  do   case $line in
          (*$Home_name=/*)   set ${line#*$Home_name=}
                             echo ${1%;}  # no trailing ;
       ;; (*setenv\ *$Home_name\ */*) set ${line#*$Home_name}
                             echo ${1%;}
       ;; esac
  done
}


Search() {
    if [[ $# -eq 0 ]]
    then set -- $(Get_dir search starting directory) || return
    fi
    Inventory before
    for  dbhome in $(kddos find $Prd $1)
    do   Add_servers $dbhome
    done
    Inventory after
}

Add_servers() { # Add all servers under 1 dbhome
    # F23012/23473 Need to fix this!!!
    Find_sid_$PR $1 | while read dbhome server reason initfile
    do   Find $server in $before && continue
         Update server=$server dbhome=$dbhome \
                dbver=$(DBver) \
                ${reason:+reason=$reason} \
                ${initfile:+reason=$initfile} \
                $Batch_parms
    done
}

Userlist() {
   # Inventory user accounts.

   typeset tempfile=/tmp/Userlist.$$
   trap "rm $tempfile 2>&-" EXIT     # Always delete file on function exit

   { # Generate userlist, store in file
     MenuHeader -n Potential $Product Owners for Disk Search
     # Fixed-length variables for display
     typeset -R2   num="( "
     typeset -L12  user=User
     typeset -L25  desc=Description
     typeset -L30  fsys=File\ System
     typeset -L   dashes=$(Userlist_line) # fix the width
     dashes="--------------------------------------------------------------------------------------------------"
     if [[ $Linux = 1 ]] ; then
        echo -e "$dashes\n$(Userlist_line)\n$dashes"
     else
        echo "$dashes\n$(Userlist_line)\n$dashes"
     fi
     num=0
     # D38215 - initialized variables
     typeset Userlist=
     # 605776:  Earlier:  Userlist_gen "$Prd|dba" | while read user fsys desc
     # "while read" into fixed-length vars cores Sol 2.6 ksh.  code-around follows:
     typeset a="" b="" c=""
     # D27033 - changed pipe/while loop to env var assign/for lopp
     #  Userlist_gen "$Prd|dba" | while read a b c
     UserList_out=`Userlist_gen "$Prd|dba"`
     OLD_IFS=$IFS
     IFS="
"
     for entry in $UserList_out
     do  
         a=`echo $entry | cut -d" " -f1`
         b=`echo $entry | cut -d" " -f2` 
         c=`echo $entry | cut -d" " -f3`
         user=$a  fsys=$b  desc=$c
         ((num+=1))
         Userlist_line
         Userlist=$Userlist\ $user
     done
     IFS=$OLD_IFS
     echo "$dashes"
     echo " $((num+=1))) | All of the above"
     echo " $((num+=1))) | Help..."
     echo "$dashes"
   } >$tempfile

   if [[ -z $Userlist ]]
   then echo "
   No users found with $Prd or dba in their names or descriptions"
        return
   fi

   # D38215 - initialized variables
   typeset ans=""
   while true
   do  echo "$(<$tempfile)"
       if ans=`Select $Userlist All Help`
       then case $ans in
             (return | Help) $ans
          ;; (All) for me in $Userlist
                   do  eval Search ~$me
                   done
          ;; (*)   eval Search ~$ans
          ;; esac
       fi
   done
}

Userlist_line() { echo "$num) | $user | $desc | $fsys"; }

Userlist_gen() {
 # Userlist string
 # Return for each user that maches string: user filesys desc.

 # translation of next statement:
 #  concatenate  /etc/passwd (has priority) with ypcat & nixcat results
 #  select lines with string (-i=ignore case)
 #  cut to only user:description
 #  sort by user (-u=unique keys, t:=delimiter,
 #    +0 -1 means key is beginning of line to next-to-last field
 #    (older form of sort key required on Sol 2.4).
## niscat passwd.org_dir 2>&-

 # D38215 - initialized variables
 typeset user="" desc="" home="" pass="" x=""
 { cat /etc/passwd
   ypcat  passwd.byname  2>&-
 } | egrep -i "$1" | sort -ut: +0 -1 |
   # F23012/23473 Need to fix this!!!
   while IFS=: read user x x x desc home x
   do   [[ -d $home ]] && echo $user $(echo Userlist_loc $home) $desc
   done
}

Userlist_loc() {
   # Return physical location of provided directory
   # On HP9 systems, /bin/df doesn't work, so customers often
   # install bdf (Berkley df) instead.  Also, various versions of df
   # have different return formats (some have headings, others don't)
   #    system:/mount
   #    /user/home/xxx  (system:/mount)
   # The following code handles all cases.  If df doesn't work, run bdf
   # instead.  If () are present, take only what is between them (if the
   # delimiter isn't present, cut passes the line through as is), then
   # get the first blank-delimited word.

   { bdf $1 || df -k $1 || echo "\n<unknown>"; } 2>&- | sed '2!d; s/ .*//'
}

Add_manual() {
   # Auto-or-manually add
   MenuHeader "Add server(s)"
   dbhome=`Get_dir $Home_name dir for server` || return

   if [[ -d $dbhome ]]
   then
      # Oracle requires dbhome as physically listed (unimplemented)

      echo "    Allowing self-discovery to fill in details..."
      Search $dbhome       # note this is a slight misuse of $dbhome
   fi

   echo "continuing with manual add..."
   echo
   server=$(Read "Enter Server name or enter to return to menu  ")
   [[ -z $server ]] && return
   dbver=`Read "Enter $Product software release level  "`
   initfile=$(Read "Enter $Init_name filename  (optional)  ")
   if  [[ $Get_login = yes ]]
   then Get_login_info $server
        Update dbhome=$dbhome server=$server \
               dbver=$dbver initfile=$initfile \
               login=$login password=$password
   else Update dbhome=$dbhome server=$server \
               dbver=$dbver initfile=$initfile
   fi
   Modify_server $HOST\_$PR\_$server.cfg

}

Login() {
  Display 'Login/Password for' Mod_login
}

Modify() {
  Display Display/Modify Modify_server
}

Display() {
   # Generic display:  Usage: Display heading-verb select-function
   typeset tempfile=$CANDLECFG/Userlist_$HOST.cfg
   trap "\rm $tempfile 2>&-" EXIT     # Always delete file on function exit
   while true
   do # Note, coding it as:  { list } > tempfile; cat tempfile
      # instead of:          { list } | cat
      # because the latter executes list in a separate process, and
      # all of its variable changes are lost.

      { MenuHeader -n $1 $Product Servers
        files=`Inventory -f`
        List $files || return
        typeset -R2 c=$List_count
        if [[ $Get_login = yes ]]
        then echo "\
    $((c+=1))) Specify the same login/password for all servers
    $((c+=1))) Specify login/password for each server in turn
$dashes"
        fi
        echo "\
    $((c+=1))) Help"
        MenuFooter Select a server to display or modify
      } > $tempfile
     cat $tempfile

     if ans=`Select $files log_all log_each`
     then case $ans in
           (log_all) Get_login_info
              for  file in $files
              do   Update $file login=$login password=$password
              done
        ;; (log_each) for file in $files
              do  Get_login_info ${file%.*}
                  Update $file login=$login password=$password
              done
        ;; (return)  return
        ;; (Help)    Help
        ;; (*)        $2 $ans   # Call function whose name passed by Login or Modify
        ;; esac
     fi
   done
}

Modify_server() {  # arg is config file name
  file=$1
  typeset -L11 init__name=$Init_name   # fix length.
  typeset -L11 home__name=$Home_name   # fix length.
  Update="Update $file"  # short-cut var
  cd $CANDLECFG
  while true; do
    . ./$file   # reread file
    MenuHeader Modify $Product server ${file%.*}
    cat <<!
        1) $home__name:  $db_home
           If this is right, version is automaticaly updated.

        2) $init__name:  $db_initfilename  (optional)
        3) Version    :  $db_ver
!
    # D38215 - initialized variables
    # IY92195  COLL_DISABLE_CURSORS setting
    typeset -i n
    typeset choices=""
    if [[ $Get_login != yes ]]
    then n=3           # as listed above
    else n=5           # plus next two
         choices=Mod_login\ Mod_pass
         if [[ -z $db_password ]]
            then db_pass_show="<not configured>"
            else db_pass_show="<encrypted>"
         fi
        if [[ $PR = or ]] then
          n=7
          cat <<!
        4) login id   :  $db_login
        5) Password   :  $db_pass_show
        6) Database TNS_ADMIN setting   :  $db_tns
        7) Extended Parameters   :  $db_extparms
!
# IY92195  COLL_DISABLE_CURSORS setting
          dbextparms=$db_extparms
        else
          cat <<!
        4) login id   :  $db_login
        5) Password   :  $db_pass_show
!
        fi
    fi
         cat <<!
        $((n+=1))) Display complete configuration details
        $((n+=1))) Help
!
    MenuFooter Modify Server options

  if [[ $PR = or ]] then
    if ans=`Select Mod_home  Mod_init Mod_ver \
                   $choices Mod_tns Mod_coll Details Help`
    then $ans
         # IZ20186 If option is Modify passwd and the new listener path is not equal to existing listener path.
         if [[ $ans = Mod_pass && $db_tns != $TNS_ADMIN ]] 
         then
             db_tns=$TNS_ADMIN                       
             Update $file dbtns="$TNS_ADMIN"   # updates the config file      
         fi                                                
    fi
  else
    if ans=`Select Mod_home  Mod_init Mod_ver \
                   $choices Details Help`
    then $ans
    fi
  fi
  done
}

Mod_home() {
   echo "  If possible, Version will be updated"
   dbhome=`Get_dir "home directory of server(s) to be modified"`
   Update $file dbhome=$dbhome \
       $(test -d $dbhome && echo dbver=$(DBver))
}

Mod_ver() {
   Update $file dbver=`Read "Enter $Product software release level  "`
}

Mod_init() {
   initfile=$(Read "Enter $Init_name filename  (fully qualified)  ")
   Update $file initfile=$initfile
}

Mod_login() {
   file=${1-$file}             # arg overrides file
   Get_login_info ${file%.*}
   Update $file login=$login password=$password
}

Mod_pass() { Update $file password=`Get_password $db_login`; }

# IY92195  COLL_DISABLE_CURSORS setting
Mod_tns() {
   Update $file dbtns=`Read "Database TNS_ADMIN setting  "`
}

# IY92195  COLL_DISABLE_CURSORS setting
Mod_coll() {
   print -n "Enter Extended Parameters  "
   read -r
   dbextparms=$REPLY
   Update $file
}

Details() {
  cat $file
  read x?"Press Enter to continue..."
}

Exclude() {
   # Toggle exclude status
   # D38215 - initialized variables
   typeset server="" status="" sure="" file="" verb=""
   while true; do
     MenuHeader $Product Inclusion Status
     files=`Inventory -a`     # All files, including excluded ones
     List $files || return
     MenuFooter Toggle Status

     if file=`Select $files`; then
        [[ $file = return ]]  && return
        server=${file##*_}
        server=${server%.cfg}
        eval status=\$db_$server   # variable-named variable set by List()
        [[ $status = Excluded ]] && verb=re-activate || verb=exclude
        sure=`Read "Are you sure you want to $verb server $server (y/n)   "`
        if [[ $sure = y ]]
        then Update $file toggle_status
        else echo "Configuration file for $server unchanged"
        fi
     fi
   done
}

List_line() {
echo "\
   $num) $db_sid_padded | $db_ver_padded | $db_installstatus_padded | $db_login_padded |"
}

List() {
   # List servers, args=list of inventory files.
   if [[ $# = 0 ]]
   then  echo "   No $Product servers configured for $HOST."
         return 99
   fi

   # D50644 - RH5/SLES10 doesn't correctly handle names that are used
   #          for global vars and also used for local vars, using typeset. Thus, 
   #          we will make the local var names unique by appending _padded
   # Define vars in config file, but fix their width for display
     typeset -R2  num="( "
     typeset -L18 db_sid_padded="Server Name"
     typeset -L7  db_ver_padded="Version"
     typeset -L20 db_installstatus_padded="Installation Status"
     typeset -L12 db_login_padded="Login"
     typeset -L   dashes=$(List_line)        # fix width=a line

   dashes="--------------------------------------------------------------------------------------------------"

   if [[ $Linux = 1 ]] ; then
      echo -e "$(List_line)\n$dashes"
   else
      echo "$(List_line)\n$dashes"
   fi
   cd $CANDLECFG
   num=0
   for file; do
       num=$((num+1))
       . ./$file  || continue        # No listing if not valid script

       # D50644 - copy the values that were set by sourcing the cfg file
       #          into special padded variables fro being displayed
       db_sid_padded=$db_sid
       db_ver_padded=$db_ver
       db_installstatus_padded=$db_installstatus
       db_login_padded=$db_login
       # Note, supporting archaic forms of db_installstatus in case
       # user doesn't have write authority to the config dir, and
       # Verify couldn't bring everything up to date.
       # (note, all vars have trailing blanks )

       case $db_installstatus in
         NOTCONFIG* | INCOMPLETE* )
                      db_installstatus_padded="Incomplete"     ;;
         DBNOTEXIS* | EXCLUDED*  )
                      db_installstatus_padded="Excluded"       ;;
         CONFIGSUC*)  db_installstatus_padded="Fully Configured" ;;
         7.1.4_NOT*)  db_installstatus_padded="7.1.4 login must=sys" ;;
       esac
       List_line

       # remember install status for caller
       eval db_${db_sid%% *}=${db_installstatus%% *}   # only first word
   done
   List_count=$num       # callers want it
   echo "$dashes"
}

Get_login_info() {
  # Get login and Password
  # D38215 - initialized variables
  typeset server=""
  test $# -eq 0 && server="servers" || server=$1
  login=`Get_login      $server`
  password=`Get_password $login`
}

Get_login() {
  login=`Read "Enter $Product connect-id for $1:    <default=candle>  "`
  echo ${login:-candle}
}

Get_password () {
  typeset sure=n
  until [ "$sure" = y ];do
 stty_org=`stty -g`
 stty -echo
     password=`Read "Enter $Product password for $1  "`
 stty $stty_org
     if [ "$password" != "" ]; then sure=y
     else sure=`Read "   Please confirm that you want a null password for $1 ?  (y/n) "`
          password=NONE
     fi
  done
  echo $password
}

Get_dir() {
    # get a directory, confirm it, convert ~dir, etc to real dir
    # args are description text.  rc=99 if abort
    typeset dir sure=n text=$*
    until [ $sure = y ]; do
       dir=`Read "     Enter $text or <CR> to return  "`
       test $dir || return 99        # caller abort

       if [[ -d $dir ]]                   # real dir or relative
         then dir=`cd $dir; echo $PWD`
              sure=y
              break
       fi

       dir2=`eval ls -d $dir`           # maybe a ~user dir
       if [[ -d $dir2 ]]
         then dir="$dir2"
              sure=y
         else echo "$dir is not a directory" 1>&2
              Beep
              sure=`Read "Confirm using directory $dir ? (y/n)  "`
       fi
    done
    echo $dir
}

Select() {
   # menu selection.  Return text answer based on number
   # Note, Read() function defined in CandleDBconfig.
   typeset -i int=-1 n=$#
   until [[ $int -ge 0 && $int -le $n ]]
   do   # Grouch:  int=`Read ....` sets int=0 if user hits enter on all
        #          Unix except rarely on aix 3.   This is a code-around.
        ans=`Read "     Please select an option or <CR> to exit: "`
        test "$ans" = "" && int=0 || int=$ans   # complain if not int.
   done
   # D38215 - initialized variables
   typeset choices=""
   set -A  choices return "$@" # if user executes result, 0=return
   echo ${choices[$int]}
}


Inventory() {
  # return an inventory of configured databases for prod
  # Following usages are mutually exclusive:
  # Usage Inventory -f  return list of non-excluded files for this host
  #       Inventory -a  return a list of all config files for this host
  #       Inventory     before/after  (excluded marked with *)
  #     Print a report of active databases for this prod, and
  #     export before or after to contain a list of servers.

  cd $CANDLECFG

  # Files for this host only.
    typeset files=$(ls $HOST\_$PR\_*.cfg 2>&-)

  case $1 in
     (-a) echo $files
  ;; (-f) [[ -n $files ]] &&
          grep -l 'db_installstatus=".[^X]' $files
  ;; (*)  # D38215 - initialized variables
          typeset server="" status="" inventory="" line=""
          ServerList=`grep -s db_installstatus $files junk`
          OLD_IFS=$IFS
          IFS="
"
          for ServerLine in $ServerList
          do 
             # D25612 - changed the following commands to tolerate FQ names.
             # D38639 - changed lines that get server name to get the name from inside the file.
             if [[ $Linux = 1 ]]
             then
                cfgFile=`echo $ServerLine | cut -d: -f1`
                server=`grep -s db_sid $cfgFile | cut -d\" -f2`
                status=`echo $ServerLine | awk -F[_:=\"] '{print $7}'`
             else
                cfgFile=`echo $ServerLine | cut -d: -f1`
                server=`grep -s db_sid $cfgFile | cut -d\" -f2`
                status=`echo $ServerLine | awk -F[_.:=\"] '{print $8}'`
             fi
             case $status in
                 (CONFIGSUCCESS) st=
              ;; (EXCLUDED)      st="(X)"
              ;; (*)             st="(I)"
              ;; esac
              inventory=$inventory${inventory:+ }$server$st
          done
          IFS=$OLD_IFS
          export $1="$inventory"
          if [[ -z $inventory ]]
          then echo "No $Product servers known $1 search."
               return 1
          else echo "\
Inventory of known $Product servers $1 search:  X=Excluded I=Incomplete
   $inventory"
          fi
  ;; esac
}

Find() {
  # Usage: Find word in blank delimited list of words
  #   If so, return 0, else return 1.  Note, words must match exactly
  word=" $1 "
  shift 2   # swallow "$word in"
  list=" $* "
  [[ $list = *$word* ]]
}

MenuHeader() {
  # D38215 - initialized variables
  typeset n=""
  # F23012/F23473 - replaced the getopts with the if-then to avoid segmentation
  # fault on Linux
  #getopts n n && shift   # -n means NoUnderlines
  if [ "$1" = "-n" ] 
  then shift
       n="n"
  fi
  Menu="$*"   # Not localized, used by Help
  typeset -L80 underscores=_____________________________________________________________________________________
  typeset date=$(date "+%m/%d/%y %H:%M")
  typeset -L$((80-${#date})) head="Candle DB config: $Menu"
  if [[ $Linux = 1 ]] ; then
     echo -e "\n$head$date"
  else
     echo "\n$head$date"
  fi

  [[ $n = n ]] || echo "$underscores"
  echo
}

MenuFooter() {
  typeset msg="$*"
  typeset -L$((36-(${#msg})/2+1)) \
          underscores=__________________________________________
  if [[ $Linux = 1 ]] ; then
     echo -e "$underscores <$msg> $underscores\n"
  else
     echo "$underscores <$msg> $underscores\n"
  fi
}

Beep() { echo '' >&2; }


DBver() { # get version from dbhome
  case $PR in
       (or)
       if [[ -a $dbhome/orainst/unix.rgs ]] then
           set -- $(fgrep ocommon $dbhome/orainst/unix.rgs)
           home=${5#\"}      # 5th word, but in quotes
           echo ${home%\"}
       elif [[ -a $dbhome/install/unix.rgs ]] then
           set -- $(fgrep rdbms $dbhome/install/unix.rgs)
           home=${2#\"}      # 2th word, but in quotes
           echo ${home%\"}
       # D44032 - try to get the version from the inventory file
       elif [[ -d $dbhome/inventory/Components21/oracle.rdbms ]] then
           echo $(ls $dbhome/inventory/Components21/oracle.rdbms)
       else
           export ORACLE_HOME=$dbhome
           echo $(korrel)
       fi
    ;; (oy) # SQL Server/10.0.2/... from dataserver  output
        # need to add lib3p or lib3p64 directory
        verString=${dbhome##*ASE-}
        ARCHBIT=`echo $CANDLEARCH | sed 's/^.*\(.\)$/\1/'`
        if [[ $AIX = 1 ]]
        then 
          if [[ `getconf KERNEL_BITMODE` = '64' ]]
          #if [[ "$ARCHBIT" = "6" ]]
          then
            export LIBPATH="$LIBPATH":$dbhome/lib:$dbhome/../OCS-$verString/lib3p64
          else
            export LIBPATH="$LIBPATH":$dbhome/lib:$dbhome/../OCS-$verString/lib3p
          fi  
        fi
        # D71070 Can't configure on Linux because of missing LD_LIBRARY_PATH
        if [[ $Solaris = 1 || $Linux = 1 ]]
        then 
          if [[ "$ARCHBIT" = "6" ]]
          then
            export LD_LIBRARY_PATH="$LD_LIBRARY_PATH":$dbhome/lib:$dbhome/../OCS-$verString/lib3p64
          else
            export LD_LIBRARY_PATH="$LD_LIBRARY_PATH":$dbhome/lib:$dbhome/../OCS-$verString/lib3p
          fi  
        fi
        $dbhome/bin/dataserver -v | awk -F/ 'NR==1 { print $2; exit 0 }'
    ;; (ox)
        $dbhome/bin/onaudit -v | awk '/Version/{print $NF; exit 0}'
    ;; (da) say "DB2 code not ready"
    ;;
  esac
}

# Function to normalize file names by removing leading irrelevant directories
# (like /tmp_mnt/).   Arg is var name.

PathNorm() {
  # D38215 - initialized variables
  typeset pathnorm_file="" short=""
  eval pathnorm_file=\$$1
  [[ $pathnorm_file = /* ]] || return   # accept only fully-qualified names

  while short=${pathnorm_file##/+([!/])}
        [[ $pathnorm_file -ef $short ]]
  do    pathnorm_file=$short
  done

  eval $1=$pathnorm_file
}

Update() {
  # Update file toggle_status keyword=value...
  # env var Force=yes means even excluded servers
  # the Product function must have been called!!!

  # Variable-localization firewall:  like-named variables
  # in calling functions must not affect Update().
  # Tolerating pipedir, although support for pipedir has
  # been removed from the rest of this.

  test $Debug && echo Update $*

  # D38215 - initialized variables
  typeset dbhome="" login="" password="" server="" dbver="" reason="" \
          initfile="" file="" pipedir="" dbhost="" syspass="" toggle=""

  typeset   db_sid="" db_home="" db_initfilename="" db_errlog="" db_login=""  \
            db_password="" db_ver="" db_type="" db_hostname="" db_pipedir=""  \
            db_installstatus="" db_reason=""

  for Arg
  do  typeset -l arg=$Arg          # lower case
      case $arg in
           (*=*)   keyword=${arg%%=*}
                   eval $keyword=${Arg#*=}
        ;; (*.*)   file=$Arg
        ;; (toggle_status) toggle=yes
        ;; esac
  done

  [[ $BATCH = batch ]] && [[ $server = $SERVER ]] && BATCH=UNDEFINED

 # if no explicit hostname given anywhere, it's me
   true ${dbhost:=$(uname -n)}
   true ${file:=$dbhost\_$PR\_$server.cfg}

  # Read the existing config file if one exists.
  # Note this relies upon the file integrity check performed in Verify()

   cd $CANDLECFG
   # if file exists, (1) check executability (if it blows, CC>0)
   #                 (2) list all env vars it makes
   [[ -f $file ]] &&
      if vars=$(typeset +x `typeset +x`  #un-export all exported
                . ./$file ; typeset +x)  #list exported per file
      then

        # Tag unsupported env vars created by the script
        # IMPORTANT: any env var created by Update must be in the following
        # list, or it will be created twice

        typeset +t `typeset +t`   #un-tag all existing tagged vars
        for var in $vars
        do  if Find $var in LOGNAME \
                     COLL_SERVERID COLL_LOGONID COLL_PASSWORD \
                     COLL_VERSION  COLL_DBVERSION             \
                     ORACLE_SID ORACLE_HOME TNS_ADMIN         \
                     DSQUERY SYBASE SYBASE_OCS                \
                     INFORMIXSERVER INFORMIXDIR ONCONFIG
            then continue
            else typeset -t $var   # tag this env var
            fi
        done
        . ./$file  # now that we know it's safe

      else bad=${file%.*}.bad
           echo $file is not a valid shell script, renaming to $bad
           mv -f $file $bad
      fi


 # Apply manual/discovered parameters
   test $server      && db_sid=$server
   test $dbhome      && db_home=$dbhome
   test $initfile    && db_initfilename=$initfile
   test $dbhost      && db_hostname=$dbhost
   test $dbver       && db_ver=$dbver
   test $reason      && db_reason=$reason
 # IY92195  COLL_DISABLE_CURSORS setting
   test $dbtns       && db_tns=$dbtns
   test -z "$dbtns"       && db_tns=$db_home/network/admin
   db_extparms=$dbextparms
   if [[ $Get_login = yes ]]
   then
   test $login       && db_login=$login
   typeset pass23="$db_login $db_sid"       # 2nd and 3rd parms for password call
   test $password    && db_password=$(kddos key $password $pass23)
   else                 db_login=*OS
                        db_password=NONE
   fi
#  check the current password.  If it is 64 bytes then it needs to be re-encrypted
   [[ ${#db_password} = 64 ]] && db_password=$(kddos key $db_password $pass23)

 # Normalize names
#  PathNorm db_home
   PathNorm db_initfilename

 # Always use internal product name
   db_type=k$PR

 # For batch:  apply syspass if appropriate
 if [[ $PR = or && $db_ver = 7.1.4* && -n $syspass ]]
 then db_login=sys
      db_password=$(kddos key $syspass $pass23)
 fi

 # For sybase12 and above
 db_home_org=$db_home
 db_home_ocs=$db_home
 db_ver_only=${db_ver%%.*}

 if [[ $PR = oy && $db_ver_only > 11 ]]
 then db_home_org=${db_home%%/ASE*}
     db_home_ocs=${db_home_org}/OCS-${db_home##*ASE-}
 fi
 
 # N/A no longer supported for db_pipedir
   [[ $db_pipedir = N/A ]] && db_pipedir=

 # Determine config status.  Note, the only external dependency is that
 # CandleDBagent searches for db_installstatus="CONFIGSUCCESS" and
 # db_host="`uname -n`".

 # Otherwise, this script completely controls these definitions

 # CONFIGSUCCESS All required parms present.
 # INCOMPLETE    A required parm is missing.
 # 7.1.4_NOT_SYS Oracle 7.1.4 server defined for some login other than
 # EXCLUDED      User has requested that no agent run on this server,
 #               and future self-discovery searches should disregard it.
 # DBNOTEXIST    Archaic form of EXCLUDED.  Must honor if read.
 # NOTCONFIGURE  Archaic form of INCOMPLETE.  No code support needed.

 # First, eliminate archaic forms
 [[ $db_installstatus = DBNOTEXIST ]] && db_installstatus=EXCLUDED
 unset COLL_VERSION_TYPE

 if   [[ $toggle = yes && $db_installstatus != EXCLUDED ]]
 then db_installstatus=EXCLUDED
 elif [[ $toggle != yes && $db_installstatus = EXCLUDED && $Force != yes ]]
 then return # No parm update on excluded servers
 elif [[ $PR = or && $db_ver = 7.1.4* && $db_login != sys ]]
 then db_installstatus=7.1.4_NOT_SYS
 elif [[ 1245 = ${db_sid:+1}${db_home:+2}${db_login:+4}${db_ver:+5} ]]
 then db_installstatus=CONFIGSUCCESS    # kddexec.sh relies on this text
 else db_installstatus=INCOMPLETE
 fi
# IY92195  COLL_DISABLE_CURSORS setting
 if [[ $PR = or && ! -f $db_tns/listener.ora ]] ; then
   echo "The listener.ora file was not found at $dbtns or the file permissions"
   echo "on the listener.ora file are restricted for this user id."
   echo "Please re-enter the path or check the file permissions..."
   db_tns="<not configured>"
   db_installstatus=INCOMPLETE
 fi

# IY92195  COLL_DISABLE_CURSORS setting
 collVars=`typeset +t`

# Write the file.  Include code to write any new
# If you add a exported variable to this shell, add it to the
# supported list above.

# Note, '#" is unspecial to cat, So $var and $(cmd) substitution work OK
#       Version depends upon CTVC Header format
#       db_pipedir supported in replication only.  No support for Update pipedir=

  cat >.$file <<! && mv -f .$file $file || echo Error creating $file, no change.
#  IBM Tivoli Monitoring for Databases: $Product Agent
#  Configuration file: $file
#  Written by CandleDBconfig version: $(set `Version`; echo $3) on $(date "+%d%b%y %T")

#  Note: this is a Korn-shell script that is "sourced" to create the 
#  environment for a Monitoring Agent for $Product.  It can be used to create 
#  the environment to run the IBM-supplied database grant scripts.

#  IBM does not recommend that you modify this file, but you can change the
#  data values in the User-defined section as long as the file is a valid ksh
#  script that executes with zero return code, and values remain quoted with "".

#  Following environment variables are set for convenience in running grants.
$(case $PR   in
   (or)      cat <<@
   export ORACLE_SID="$db_sid"
   export ORACLE_HOME="$db_home"
   export TNS_ADMIN="$db_tns"
@
;; (oy)      cat <<@
   export DSQUERY="$db_sid"
   export SYBASE="$db_home_org"
   export SYBASE_OCS="$db_home_ocs"
@
;; (ox)      cat <<@
   export INFORMIXSERVER="$db_sid"
   export INFORMIXDIR="$db_home"
   export ONCONFIG="${db_initfilename##*/}"    # ONCONFIG must be a bare filename
@
;; esac)

#  Following variables are set for the setup scripts
   db_sid="$db_sid"
   db_home="$db_home"
   db_initfilename="$db_initfilename"
   db_login="$db_login"
   db_password="$db_password"
   db_ver="$db_ver"
   db_type="$db_type"
$(case $PR   in
   (or)      cat <<@
   db_tns="$db_tns"
   db_extparms="$db_extparms"
@
;; esac)
   db_hostname="$db_hostname"
   db_reason="$db_reason"
   db_pipedir="$db_pipedir"
   db_installstatus="$db_installstatus"

#  User-defined environment variables
$( if [[ $PR = oy && $collVars != *COLL_HOLD_SYSMON* ]] then
     echo "#  To hold sp_sysmon collection: COLL_HOLD_SYSMON=1"
     echo "#  Default value of COLL_HOLD_SYSMON is 0\n"
     echo "      export COLL_HOLD_SYSMON=\"0\""
   fi )
$( for var in `typeset +t`
   do  eval val=\$$var
     if [[ $PR = oy && $var = COLL_HOLD_SYSMON ]] then
       echo "#  To hold sp_sysmon collection: COLL_HOLD_SYSMON=1"
       echo "#  Default value of COLL_HOLD_SYSMON is 0"
     fi
     echo "   export $var=\"$val\""
   done )
!

  [[ $BATCH = UNDEFINED ]] && [[ ! -z $db_home ]] && BATCH=$db_installstatus

}

Help() {
  # I'm only called if kddhelp.sh hasn't yet defined the REAL Help function
  if   . kddhelp.sh
  then Help $*
  else echo "Help non-functional"
  fi
}


Batch() { # Non-interactive driver.   Non-operational for now.
  test $Debug && say Batch "$*"
  # Hierarchy of args:
  # Product      required.
  # Only one of the following three:
  #   scan       Scan running jobs, /etc/oratab
  #   search=    Disk search from starting points.  wildcards OK,
  #              muliple starting points OK if quoted.
  #   modify=    Servers to modify, must be quoted if more than one.
  #              modify=* means all unexcluded servers for this host
  #              No other wildcarding allowed
  #
  #      keyword=value   dbhome initfile dbhost dbver pipedir login password
  #      syspass=        If Oracle 7.1.4, and syspass specified, then
  #                      login=sys, password=$syspass for that Oracle.
  #                      Allows you to configure all Oracle with a
  #                      single call.
  #
  #      toggle_status   Caution: use only with specific server.
  #
  #   Notes:
  #   If you specify any of dbhome, initfile or dbver with a search
  #   or scan call, those values override anything discovered.

  Batch=1
  Product $1             # Set Prod, Product, Prd.  See Product() for defs
  action=$2 shift 2
  Batch_parms=$@
  case $action in
       (scan)     Scan
    ;; (search=*) for search in ${action#search=}
                  do  Search $search
                  done
    ;; (modify=*) if [[ ${action#modify=} = * ]]
                  then servers=`Inventory -f`
                  else servers=${action#modify=}
                  fi
                  for server in $servers
                  do  Update server=${server%.*} $Batch_parms
                  done
    ;; esac
  List `Inventory -a`
}

say() { if [[ $Linux = 1 ]] then echo -e "$*" >&2; else echo "$*" >&2; fi }


######################################################################
#                              MAIN PROGRAM                          #
######################################################################

  BATCH=interactive
  SERVER=

  while getopts :dls:i:p:VL opt
  do case $opt in
        (d) Debug=debug
            read x?"-d option writes passwords to the log file! (enter)"
     ;; (l) New_log=yes
     ;; (V) Verify=no
     ;; (L) Logfile=/dev/null
     ;; (s) BATCH=batch # command line
            SERVER=$OPTARG
     ;; (i) BATCH=batch # command line
            ID=$OPTARG
     ;; (p) BATCH=batch # command line
            PW=$OPTARG
     ;; (*) echo   "$PROGNAME failure: invalid option -$OPTARG"
           exit 1
     ;; esac
  done
  shift OPTIND-1

  true ${Logfile=$CANDLELOG/$HOST\_dd_config.log}
  [[ $New_log = yes ]] && rm $Logfile 2>&-

# Call Select_product.   Redirections:
# 1 stdout               piped to "tee"
# 2 stderr               redirected to stdout to pipe to tee
# 3 "real stderr"        preserved as file descriptor 3 so that
#                        more can get to it.
#
# more only works with virgin stderr and stdout>a real terminal
# cheat: Sinix does not tolerate more -e.

  rc=0
  if [[ $BATCH = batch ]]
  then PR=$1
       Get_login=yes
       Update server=$SERVER login=$ID password=$PW >>$Logfile 3>&2 2>&1
       [[ $BATCH = batch ]] || [[ $BATCH = CONFIGSUCCESS ]] || rc=99
       if (( rc > 0 ))
       then
          BATCH=batch
          rc=0
          Scan=1
          Product_setup $PR
          Scan
          Update server=$SERVER login=$ID password=$PW >>$Logfile 3>&2 2>&1
          [[ $BATCH = CONFIGSUCCESS ]] || \
          if [[ $BATCH = UNDEFINED ]]
             then rc=99 
             else rc=12
          fi
       fi
  else Select_product "$@" 3>&2 2>&1 | tee -ai $Logfile
       echo "CandleDBConfig: installation log is $Logfile"
  fi

  return $rc
